#include <utility.h>
#include <userint.h>

/*****************************************************************************/
/*   Copyright 1998 National Instruments Corporation.  All Rights Reserved.  */
/*****************************************************************************/


/*================ R3267/73 Spectrum Analyzer OPT65(CDMA2000)================*/
/* LabWindows/CVI 5.0 Instrument Driver										 */
/* Original Release: October/30/2000										 */
/* By: Ai Onishi, Hitoshi Takahashi											 */
/*					 Supervisor: Yohei Hirakoso								 */
/*                   SPSpj Gyoda R&D,  ADVANTEST CORPORATION				 */
/*                   Home Page; http://www.advantest.co.jp					 */
/*                   e-mail;    info@advantest.co.jp						 */
/*---------------------------------------------------------------------------*/
/* Modification History: 													 */
/*         Version 1.0 :  October/30/2000									 */
/*																			 */
/*===========================================================================*/
 

#include <visa.h>
#include <ansi_c.h>
#include <formatio.h>
#include "avopt65.h"

#define avopt65_REVISION    "Rev 1.0, 10/00, CVI 5.0" /*  Instrument driver revision */
#define BUFFER_SIZE         1024L 					  /* File I/O buffer size */
    

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/

/****
 ****  Opt65 Static Command Arr
 ****/

/* CDMA2000 Started Setting */
static ViString linkArr[] = {"LINK REV34","LINK FWD"};
static ViString signalRateArr[] = {"RATE 9600","RATE 4800","RATE 2400","RATE 1200"};
static ViString freqInputArr[]={"FINPMD CHL","FINPMD FREQ"};
static ViString iqInverseArr[]={"IQMD INV","IQMD NORM"};
static ViString autoLevelArr[] = {"ALS OFF","ALS ON"}; 

/* Channel Setting */
static ViString editLinkArr[]={"CHEDRV", "CHEDFR"};

/* Channel Enable */
static ViString enableArr[]={"DSBL","ENBL"};     

/* Trigger */
static ViString trgWFSrcArr[] = {"TRGSRC FREE","TRGSRC VIDEO","TRGSRC IF","TRGSRC EXT"};
static ViString trgSlopeArr[] = {"TRGSLP FALL","TRGSLP RISE"};

/* Gate */
static ViString trgFdpSrcArr[] = {"FREE","VIDEO","IF","EXT"};
static ViString gateExSTDArr[] = {"DTMSETSTD","FDPSETSTD"};

/* Window set */
static ViString switchArr[] = {"OFF","ON"};       

/* Template Edit */
static ViString UpLowArr[]={"LOW","UP"};

/* Detecter Select */
static ViString DetArr[]={"DET NRM","DET POS","DET NEG","DET SMP"};

/* Select Table 1 to 3 */
static ViString selTblArr[] = {"TBL 1","TBL 2","TBL 3"};

/* Table Frequency Type */   
static ViString tableFArr[] = {"TBLF REL","TBLF ABS"};

/*TDS Param*/
static ViString resultArr[] = {"RES RMS","RES PK"};
static ViString preselArr[]={"PRE 36G", "PRE 16G"};

/* DTS Param */
static ViString FreqSetArr[] ={"FRMD CFSP","FRMD STSP"};
static ViString LvlUnitArr[] ={"UNIT DBM", "UNIT W", "UNIT DBUV"};
static ViString dtsResultArr[] = {"RES MKR", "RES REL","RES ABS"};
static ViString refMkrArr[] = {"REF MOD","REF MKR"};

/* Modulation */
static ViString trgModeArr[] = {"TRG INT", "TRG EXT", "TRG INTRVL1", "TRG INTRVL2"};

/* CDP Coefficient */
static ViString sclFmtArr[] = {"C2PCFMT GRP", "C2PCFMT TBL", "C2PCFMT NUM"};
static ViString sclDispArr[] = {"C2PCDISP DUAL", "C2PCDISP SNGL"};
static ViString sclYCDPCFArr[] = {"RHO", "POW", "TAU", "PHA", "CDE"};
static ViString yDivArr[] = {"C2PDIV P5", "C2PDIV P10"};
static ViString chnlNameArr[] = {"C2CHNAME PICH", "C2CHNAME SYNCH", "C2CHNAME PCH",
								 "C2CHNAME BCH", "C2CHNAME CCCH ", "C2CHNAME DCCH",
								 "C2CHNAME FCH", "C2CHNAME SCCH", "C2CHNAME SCH"};
static ViString grphPltTypArr[] = {"C2CDPLOT AVG", "C2CDPLOT MAX", "C2CDPLOT MIN"};    
static ViString pwrUnitCDPArr[] = {"C2PWRUNIT DBM", "C2PWRUNIT DB", "C2PWRUNIT DBMPI"};
static ViString crrFreqSrchArr[] = {"C2FSRCH 500HZ", "C2FSRCH 10KHZ", "C2FSRCH 500KHZ"};

/* CDP */
static ViString msModeCDPArr[] = {"C2CDMMOD NORM", "C2CDMMOD PREC"};
static ViString freqMsRngArr[] = {"C2FRRNG EXP", "C2FRRNG NORM"};
static ViString dispCDPArr[] = {"C2DISP RHO", "C2DISP EVM", "C2DISP DEMOD", 
                                "C2DISP NUM", "C2DISP TBL"};
static ViString viewChnlArr[] = {"PICH", "DCCH", "SCH2", "FCH", "SCH1", "EACH", "CCCH"};
static ViString oprModeArr[] = {"C2OP TCH", "C2OP EACH", "C2OP CCCH"};
static ViString sch1WalshArr[] = {"C2SCH1WALSH W24", "C2SCH1WALSH W12"};
static ViString sch2WalshArr[] = {"C2SCH2WALSH W68", "C2SCH2WALSH W24"};  

/* Tx Power */
static ViString TXtrigSourceArr[] = {"TTRGSRC INT","TTRGSRC EXT","TTRGSRC INTRVL1"};

/* CCDF */
static ViString pwrUnitArr[] = {"UNIT ABS", "UNIT REL"};
static ViString trgMdCCDFArr[] = {"C2CCDFTRG EXT", "C2CCDFTRG INT"};

/* Avarage */
static ViString avgFunctionArr[] = {"TDPAVG", "OORAVG", "TDSAVG", "FDPAVG", "OBWAVG",
									"DTSAVG", "DTMAVG", "SPRAVG", "FDSAVG", "C2CDAVG",
									"TXAVG"};

/* Y Scale */
static ViString yScaleFunctionArr[] = {"TDPDIV", "OORDIV", "FDPDIV"};
static ViString yScaleArr[] = {"P10DB", "P5DB", "P2DB"};

/* Auto Level */
static ViString autoLvlFunctionArr[] = {"TDPAUTOLVL","OORAUTOLVL","TDSAUTOLVL","FDPAUTOLVL",
										"OBWAUTOLVL", "DTSAUTOLVL", "DTMAUTOLVL", "SPRAUTOLVL",
										"FDSAUTOLVL", "AUTOLVL"};

/* Mesurement Start */
static ViString funcMeasureArr[] = {"TDPMEAS", "OORMEAS", "TDSMEAS", "FDPMEAS", "OBWMEAS", "DTSMEAS",
									"DTMMEAS", "SPRMEAS", "FDSMEAS",
									"C2PC", "C2CDP", "TXPWR", "C2CCDF", "SI"};
									
/* Graph Type Select */
static ViString slctGraphArr[] = {"GTYP CON", "GTYP CONLIN", "GTYP CONDOT",
								  "GTYP CONLINDOT", "GTYP ICHEYE", "GTYP QCHEYE",
								  "GTYP IQCHEYE", "GTYP EVM", "GTYP MAGERR"};
static ViString plotTypeArr[] = {"GPLOT PP", "GPLOT AVG"};				  

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ============================*/
/*****************************************************************************/
/* avopt65_stringValPair is used in the avopt65_errorMessage function        */
/* avopt65_statusDataRanges is used to track session dependent status & ranges*/
/*===========================================================================*/
typedef struct  avopt65_stringValPair
{
   ViStatus stringVal;
   ViString stringName;
}  avopt65_tStringValPair;
 
struct avopt65_statusDataRanges {
    ViInt16 traceRange;
    ViInt16 harmonics;
    ViChar instrDriverRevision[256];
};

typedef struct avopt65_statusDataRanges *avopt65_instrRange;

/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
ViBoolean avopt65_invalidViBooleanRange (ViBoolean val);
ViBoolean avopt65_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max);
ViBoolean avopt65_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean avopt65_invalidViUInt8Range (ViUInt8 val, ViUInt8 min, ViUInt8 max);
ViBoolean avopt65_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max);
ViBoolean avopt65_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max);
ViBoolean avopt65_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max);
ViBoolean avopt65_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);
ViStatus avopt65_waitOnVisaEvent (ViSession instrSession, ViEventType eventTypeIn,
	ViUInt32 tmoIn, ViPUInt16 STB);
ViStatus avopt65_initCleanUp (ViSession openRMSession, ViPSession openInstrSession,
	ViStatus currentStatus);
ViStatus avopt65_defaultInstrSetup (ViSession openInstrSession);
ViStatus avopt65_dataGraph (ViSession instrumentHandle, ViInt16 grphSlct,
                            ViInt16 *iDataSize, ViReal64 iData[],
                            ViInt16 *qDataSize, ViReal64 qData[],
                            ViInt16 *xDataSize, ViInt16  xData[],
                            ViInt16 *yDataSize, ViReal64 yData[]);

/*****************************************************************************/
/*====== USER-CALLABLE FUNCTIONS (Exportable Functions) =====================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Standard Setting                                                */
/* Purpose:  This function sets the communication type and link of the signal*/
/*           under TRANSIENT measurement.									 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt65_confStdSet (ViSession instrumentHandle,
                                      ViInt16 bandClass, ViBoolean link,
                                      ViReal64 offsetLevel, ViBoolean freqInput,
                                      ViBoolean iqInverse, ViBoolean contAutoLevel)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;
    	
    if (avopt65_invalidViInt16Range (bandClass, 0, 7))
        return VI_ERROR_PARAMETER2;
    if (avopt65_invalidViBooleanRange (link))
        return VI_ERROR_PARAMETER3;
    if (avopt65_invalidViReal64Range (offsetLevel, -100.0, 100.0))
        return VI_ERROR_PARAMETER4;
    if (avopt65_invalidViBooleanRange (freqInput))
        return VI_ERROR_PARAMETER5;
    if (avopt65_invalidViBooleanRange (iqInverse))
        return VI_ERROR_PARAMETER6;
    if (avopt65_invalidViBooleanRange (contAutoLevel))
        return VI_ERROR_PARAMETER7;
        
    Fmt (wrtBuf, "%s<BNDCLS %d; %s; RO %fDB; %s; %s; %s", bandClass, linkArr[link],
    		offsetLevel, freqInputArr[freqInput], iqInverseArr[iqInverse],
    		autoLevelArr[contAutoLevel]);
    
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;       
    		
}

/*===========================================================================*/
/* Function: CDMA Channel Setting                                            */
/* Purpose: This function edits channel setting.                             */
/*               						                                	 */
/*===========================================================================*/

ViStatus CVIFUNC avopt65_ChEdt (ViSession instrumentHandle, ViBoolean settoSTD,
                                ViBoolean editLink, ViInt16 editChnl,
                                ViInt16 startCHNum, ViInt16 stopCHNum, 
                                ViReal64 baseFreq, ViReal64 channelSpace, 
                                ViInt16 chOffset)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if (avopt65_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;
    if (avopt65_invalidViBooleanRange (editLink))
        return VI_ERROR_PARAMETER3;
    if (avopt65_invalidViInt16Range (editChnl, 1, 9))
        return VI_ERROR_PARAMETER4;
    if (avopt65_invalidViInt16Range (startCHNum, -12800, 12800))
        return VI_ERROR_PARAMETER5;
    if (avopt65_invalidViInt16Range (stopCHNum, -12800, 12800))
        return VI_ERROR_PARAMETER6;
    if (avopt65_invalidViReal64Range (baseFreq, 0.0, 100000000000.0))
        return VI_ERROR_PARAMETER7;
    if (avopt65_invalidViReal64Range (channelSpace, -100000000.0, 100000000.0))
        return VI_ERROR_PARAMETER8;
    if (avopt65_invalidViInt16Range (chOffset, -12800, 12800))
        return VI_ERROR_INSTR_PARAMETER9;

	if (settoSTD == VI_ON)
		Fmt (wrtBuf, "%s<CHSETSTD");
	else
		Fmt (wrtBuf, "%s<%s%d %d,%d,%fHZ,%fHZ,%d", editLinkArr[editLink], editChnl,
    			startCHNum, stopCHNum, baseFreq, channelSpace, chOffset);

    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;  
   
}


/*===========================================================================*/
/* Function: Channel Enable ON/OFF                                           */
/* Purpose:  This function sets the state of Channel table enable.           */
/*                                      									 */
/*===========================================================================*/

ViStatus CVIFUNC avopt65_ch_Enable (ViSession instrumentHandle,
                                    ViBoolean channel1, ViBoolean channel2,
                                    ViBoolean channel3, ViBoolean channel4,
                                    ViBoolean channel5, ViBoolean channel6,
                                    ViBoolean channel7, ViBoolean channel8,
                                    ViBoolean channel9)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if (avopt65_invalidViBooleanRange (channel1))
        return VI_ERROR_PARAMETER2;	
    if (avopt65_invalidViBooleanRange (channel2))
        return VI_ERROR_PARAMETER3;	   
    if (avopt65_invalidViBooleanRange (channel3))
        return VI_ERROR_PARAMETER4;	           
    if (avopt65_invalidViBooleanRange (channel4))
        return VI_ERROR_PARAMETER5;	
    if (avopt65_invalidViBooleanRange (channel5))
        return VI_ERROR_PARAMETER6;	   
    if (avopt65_invalidViBooleanRange (channel6))
        return VI_ERROR_PARAMETER7;	           
    if (avopt65_invalidViBooleanRange (channel7))
        return VI_ERROR_PARAMETER8;	
    if (avopt65_invalidViBooleanRange (channel8))
        return VI_ERROR_INSTR_PARAMETER9;	   
    if (avopt65_invalidViBooleanRange (channel9))
        return VI_ERROR_INSTR_PARAMETER10;	           
	
	Fmt (wrtBuf, "%s<CHTBL1 %s; CHTBL2 %s; CHTBL3 %s; CHTBL4 %s; CHTBL5 %s; CHTBL6 %s; CHTBL7 %s; CHTBL8 %s; CHTBL9 %s",
			enableArr[channel1], enableArr[channel2], enableArr[channel3],
			enableArr[channel4], enableArr[channel5], enableArr[channel6],
			enableArr[channel7], enableArr[channel8], enableArr[channel9]);
    
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;      
}

/*===========================================================================*/
/* Function: TDP Trigger Conditions                                          */
/* Purpose:  This function is used for time domain Power. This               */
/*           function sets the trigger parameters.							 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_confTrgConTDP (ViSession instrumentHandle,
                                        ViInt16 triggerSource,
                                        ViBoolean trgSlope, 
                                        ViInt16 triggerLevel,
                                        ViInt16 triggerPosition,
                                        ViReal64 delayTimes)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if (avopt65_invalidViInt16Range (triggerSource, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (avopt65_invalidViBooleanRange (trgSlope))
        return VI_ERROR_PARAMETER3;
    if (avopt65_invalidViInt16Range (triggerLevel, 0, 100))
        return VI_ERROR_PARAMETER4;
    if (avopt65_invalidViInt16Range (triggerPosition, 0, 100))
        return VI_ERROR_PARAMETER5;
    if (avopt65_invalidViReal64Range (delayTimes, -1000.0, 1000.0))
        return VI_ERROR_PARAMETER6;

    Fmt (wrtBuf, "%s<%s;%s;TRGLVL %dENT", trgWFSrcArr[triggerSource],
    	trgSlopeArr[trgSlope], triggerLevel);
    total += NumFmtdBytes();
	if (triggerSource > 0) {
	    Fmt (wrtBuf, "%s[a]<;TRGPOS %dENT;TRGDT %f[e]SC", triggerPosition, delayTimes); /* Append */
	}
    total += NumFmtdBytes();
    
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;       
    
}

/*===========================================================================*/
/* Function: TDP Window Setup					                             */
/* Purpose:  This function is used for time domain Power.					 */
/*           This function sets the Window setup parameters.				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_windowSetTDP (ViSession instrumentHandle,
                                       ViBoolean settoSTD, ViBoolean window,
                                       ViReal64 winPosS, ViReal64 winWidS)
{
	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;    

    if (avopt65_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;
    if (avopt65_invalidViBooleanRange (window))
        return VI_ERROR_PARAMETER3;
    if (avopt65_invalidViReal64Range (winPosS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER4;   
    if (avopt65_invalidViReal64Range (winWidS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER5; 

    if ( settoSTD == VI_ON )
    {
        Fmt (wrtBuf, "%s<TDPSETSTD");
    	total += NumFmtdBytes();
    }
    else
    {
        Fmt (wrtBuf, "%s<TDPWDO %s; TDPWPOS %f[e]SC; TDPWWID %f[e]SC",
        	switchArr[window],
    		winPosS, winWidS);
    	total += NumFmtdBytes();
    }
               
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;    
}


/*===========================================================================*/
/* Function: TDP Template Setup					                             */
/* Purpose:  This function is used for time domain Power.					 */
/*           This function sets the Template setup parameters.				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_template_Set_TDP (ViSession instrumentHandle,
                                           ViBoolean template, ViReal64 shiftXS,
                                           ViReal64 shiftYdB)
{
  	ViUInt32 i, retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;    

	if (avopt65_invalidViBooleanRange (template))
        return VI_ERROR_PARAMETER2;

    if (avopt65_invalidViReal64Range (shiftXS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER3;   
    if (avopt65_invalidViReal64Range (shiftYdB, -200.0, 100.0))
        return VI_ERROR_PARAMETER4; 

    Fmt (wrtBuf, "%s<TDPTMPL %s; TDPTMPLSX %f[e]SC; TDPTMPLSY %fDB", switchArr[template],
    	shiftXS, shiftYdB);
    
    total += NumFmtdBytes();
    
	if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;      
}

/*===========================================================================*/
/* Function: TDP Template Edit					                             */
/* Purpose:  This function is used for Time domain Power.					 */
/*           This function sets the Table data of Template.					 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_template_editTDP (ViSession instrumentHandle,
                                           ViInt16 operation,
                                           ViBoolean templateSelect,
                                           ViReal64 templateTimes,
                                           ViReal64 templateLevel)
{
  	ViUInt32 i, retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;    

	if (avopt65_invalidViInt16Range (operation,0, 2))
        return VI_ERROR_PARAMETER2;
	if (avopt65_invalidViBooleanRange (templateSelect))
        return VI_ERROR_PARAMETER3;
 
	switch (operation)
	{
	case 0: /* Copy from STD */
		Fmt (wrtBuf, "%s<TDPTMPLCP");
		break;
	
	case 1: /* entry */
		if (avopt65_invalidViReal64Range (templateTimes, -1.0, 1.0))
        	return VI_ERROR_PARAMETER4;

		Fmt (wrtBuf, "%s<TDPTMPLSEL %s; TDPTMPLED %f[e]SC,%f[e]", 
			 UpLowArr[templateSelect], templateTimes, templateLevel);
		break;
		
	case 2: /* Table init */
		 Fmt(wrtBuf, "%s<TDPTMPLSEL %s; TDPTMPLCLR", UpLowArr[templateSelect]); 
		break;
	}
	
	if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;    
}

/*===========================================================================*/
/* Function: TDP Parameter Setup				                             */
/* Purpose:  This function is used for Time domain Power.					 */
/*           This function sets the parameters for Measurement.				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_parameter_setTDP (ViSession instrumentHandle,
                                           ViBoolean settoSTD, ViInt16 detector,
                                           ViInt16 displayUnit,
                                           ViBoolean TMPLCoupleofPWR,
                                           ViReal64 templateLimit,
                                           ViBoolean judgement,
                                           ViReal64 upperLimit,
                                           ViReal64 lowerLimit)
{
	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;    

    if (avopt65_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;


    if ( settoSTD == VI_ON )
    {
        Fmt (wrtBuf, "%s<TDPSETSTD");
    }
    else
    {
    	if (avopt65_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER3;
    	if (avopt65_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER4;
    	if (avopt65_invalidViBooleanRange (TMPLCoupleofPWR))
        	return VI_ERROR_PARAMETER5;
   		if (avopt65_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER6;        
    
    
        Fmt (wrtBuf, "%s<TDP%s; TDP%s; TDPTMPLPW %s; TDPTMPLBTM %fDB; TDPJDG %s; TDPJDGUP %fDB; TDPJDGLOW %fDB",
        	DetArr[detector], LvlUnitArr[displayUnit], switchArr[TMPLCoupleofPWR],
        	templateLimit, switchArr[judgement], upperLimit,  lowerLimit);

    }
    	total += NumFmtdBytes();                 
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;        
}


/*===========================================================================*/
/* Function: On/Off Raito Trigger   				                         */
/* Purpose:  This function is used for On/Off Raito (Time Domain).			 */
/*           This function sets the parameters for Trigger. 				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_trigger_setOOR (ViSession instrumentHandle,
                                         ViInt16 triggerSource,
                                         ViBoolean trgSlope,
                                         ViInt16 triggerLevel,
                                         ViInt16 triggerPosition,
                                         ViReal64 delayTimes)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if (avopt65_invalidViInt16Range (triggerSource, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (avopt65_invalidViBooleanRange (trgSlope))
        return VI_ERROR_PARAMETER3;
    if (avopt65_invalidViInt16Range (triggerLevel, 0, 100))
        return VI_ERROR_PARAMETER4;
    if (avopt65_invalidViInt16Range (triggerPosition, 0, 100))
        return VI_ERROR_PARAMETER5;
    if (avopt65_invalidViReal64Range (delayTimes, -1000.0, 1000.0))
        return VI_ERROR_PARAMETER6;

    Fmt (wrtBuf, "%s<OOR%s; OOR%s; OORTRGLVL %dENT", trgWFSrcArr[triggerSource],
    	trgSlopeArr[trgSlope], triggerLevel);
    total += NumFmtdBytes();
    
	if (triggerSource > 0) {
	    Fmt (wrtBuf, "%s[a]<;OORTRGPOS %dENT; OORTRGDT %f[e]SC", triggerPosition, delayTimes);
    	total += NumFmtdBytes(); 
	}

    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;       
    	    
}

/*===========================================================================*/
/* Function: On/Off Raito Window						                     */
/* Purpose:  This function is used for time domain wave form analyzing. This */
/*           function sets the trigger parameters.							 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_window_setOOR (ViSession instrumentHandle,
                                        ViBoolean settoSTD, ViBoolean window,
                                        ViReal64 onPositionS, ViReal64 onWidthS,
                                        ViReal64 offPositionS,
                                        ViReal64 offWidthS)
{
	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;    

    if (avopt65_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;
    if (avopt65_invalidViBooleanRange (window))
        return VI_ERROR_PARAMETER3;
    if (avopt65_invalidViReal64Range (onPositionS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER4;   
    if (avopt65_invalidViReal64Range (onWidthS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER5; 
    if (avopt65_invalidViReal64Range (offPositionS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER6;   
    if (avopt65_invalidViReal64Range (offWidthS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER7; 
        
    if ( settoSTD == VI_ON )
    {
        Fmt (wrtBuf, "%s<OORSETSTD");
    }
    else
    {
        Fmt (wrtBuf, "%s<OORWDO %s; OORWONPOS %f[e]SC; OORWONWID %f[e]SC; OORWOFPOS %f[e]SC; OORWOFWID %f[e]SC",
        	switchArr[window],onPositionS,onWidthS,offPositionS,offWidthS);
    }
    	total += NumFmtdBytes();           
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;    	    
}

/*===========================================================================*/
/* Function: On/Off Raito Parameter Setup				                     */
/* Purpose:  This function is used for Time domain Power.					 */
/*           This function sets the parameters for Measurement.				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_param_setOOR (ViSession instrumentHandle,
                                       ViBoolean settoSTD, ViInt16 detector,
                                       ViInt16 displayUnit, ViBoolean judgement,
                                       ViReal64 upperLimit)
{
	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;    

    if (avopt65_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;


    if ( settoSTD == VI_ON )
    {
        Fmt (wrtBuf, "%s<OORSETSTD");
    	total += NumFmtdBytes();
    }
    else
    {
    	if (avopt65_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER3;
    	if (avopt65_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER4;
   		if (avopt65_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER5;        
    
    
        Fmt (wrtBuf, "%s<OOR%s; OOR%s; OORJDG %s; OORJDGUP %fDB",
        	DetArr[detector], LvlUnitArr[displayUnit], switchArr[judgement], upperLimit);
    	total += NumFmtdBytes();
    }
               
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;    
}
/*===========================================================================*/
/* Function: TDS Trigger Set	                                             */
/* Purpose:  This function is used for time domain Spurious. This            */
/*           function sets the trigger parameters.							 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_trigger_setTDS (ViSession instrumentHandle,
                                         ViInt16 triggerSource,
                                         ViBoolean trgSlope,
                                         ViInt16 triggerLevel,
                                         ViInt16 triggerPosition,
                                         ViReal64 delayTimes)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if (avopt65_invalidViInt16Range (triggerSource, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (avopt65_invalidViBooleanRange (trgSlope))
        return VI_ERROR_PARAMETER3;
    if (avopt65_invalidViInt16Range (triggerLevel, 0, 100))
        return VI_ERROR_PARAMETER4;
    if (avopt65_invalidViInt16Range (triggerPosition, 0, 100))
        return VI_ERROR_PARAMETER5;    
    if (avopt65_invalidViReal64Range (delayTimes, -1000.0, 1000.0))
        return VI_ERROR_PARAMETER6;

    Fmt (wrtBuf, "%s<TDS%s;TDS%s", trgWFSrcArr[triggerSource],
    	trgSlopeArr[trgSlope]);
    total += NumFmtdBytes();
    
    
	if (triggerSource != 0 ) {
		
		if ((triggerSource == 1) || (triggerSource == 2)) {
	    	Fmt (wrtBuf, "%s[a]<;TDSTRGLVL %dENT", triggerLevel);
	    	total += NumFmtdBytes();		
		}
		
	    Fmt (wrtBuf, "%s[a]<;TDSTRGPOS %dENT;TDSTRGDT %f[e]SC", triggerPosition, delayTimes);
	    total += NumFmtdBytes();     
	}

    
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;       
}

/*===========================================================================*/
/* Function: TDS Table Edit						                     		 */
/* Purpose:  This function is used for Time domain Spurious.				 */
/*           This function sets the Table for Measurement.					 */
/*===========================================================================*/

ViStatus CVIFUNC avopt65_table_editTDS (ViSession instrumentHandle,
                                        ViInt16 operation, ViInt16 tableNumber,
                                        ViBoolean freqType, ViReal64 carrierFreq,
                                        ViReal64 frequency, ViReal64 level)
{
    ViUInt32 i,retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;
    
	if (avopt65_invalidViInt16Range (operation, 0, 6))
        return VI_ERROR_PARAMETER2;    
	if (avopt65_invalidViInt16Range (tableNumber, 1, 3))
        return VI_ERROR_PARAMETER3;

/*

	0: Load Table
	1: Save Table
	2: Select Table (Default value)
	3: Edit Table
	4: Delete Table
	5: Init Table
	6: Table Freq Type & Carrer Freq Input

*/

	switch (operation)
	{
	case 0: /* LOAD TABLE */
		Fmt (wrtBuf, "%s<RCL%s", selTblArr[tableNumber-1]);
    	total += NumFmtdBytes();
		break;
	case 1: /* SAVE TABLE */
		Fmt (wrtBuf, "%s<SVS%s", selTblArr[tableNumber-1]);
    	total += NumFmtdBytes();
		break;
	case 2: /* SELECT TABLE ONLY */
		Fmt (wrtBuf, "%s<TDS%s", selTblArr[tableNumber-1]);
    	total += NumFmtdBytes();
		break;
	case 3: /* DATA ENTRY */
		Fmt (wrtBuf, "%s<TDSTBLED %fHZ,%fDB",frequency, level);
    	total += NumFmtdBytes();
		break;
	case 4: /* DATA DELETE */
		Fmt (wrtBuf, "%s<TDSTBLDEL");
    	total += NumFmtdBytes();
		break;
	case 5: /* INIT TABLE */
		Fmt (wrtBuf, "%s<TDSCLR");
    	total += NumFmtdBytes();
		break;
	case 6: /* TABLE FREQ & CARRER FREQ */
		if (avopt65_invalidViBooleanRange (freqType))
        	return VI_ERROR_PARAMETER4;
        if (avopt65_invalidViReal64Range (carrierFreq, 0.0, 8000000000.0))
        	return VI_ERROR_PARAMETER5;	
		Fmt (wrtBuf, "%s<TDS%s; CF %fHZ",tableFArr[freqType],carrierFreq);
		break;
	}
	total += NumFmtdBytes();
	if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        	return avopt65_status;    
    return avopt65_status;     
}

/*===========================================================================*/
/* Function: TDS Parameter Setup				                     		 */
/* Purpose:  This function is used for Time domain Spurious.				 */
/*           This function sets the parameters for Measurement.				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_param_setTDS (ViSession instrumentHandle,
                                       ViBoolean settoSTD, ViInt16 detector,
                                       ViBoolean result, ViReal64 peakMKRYDelta,
                                       ViReal64 multiplier, ViInt16 displayUnit,
                                       ViBoolean judgement, ViBoolean preselecter)
{
	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;    

    if (avopt65_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

    if ( settoSTD == VI_ON )
    {
        Fmt (wrtBuf, "%s<TDSSETSTD");
    	total += NumFmtdBytes();
    }
    else
    {
    	if (avopt65_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER3;
	    if (avopt65_invalidViBooleanRange (result))
        	return VI_ERROR_PARAMETER4;    
   		if (avopt65_invalidViReal64Range (peakMKRYDelta, 0.1, 10.0))
        	return VI_ERROR_PARAMETER5;
        if (avopt65_invalidViReal64Range (multiplier, 1.0, 100.0))
        	return VI_ERROR_PARAMETER6;
    	if (avopt65_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER7;
   		if (avopt65_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER8;        
           if (avopt65_invalidViBooleanRange (preselecter))
        return VI_ERROR_INSTR_PARAMETER9;
        
        Fmt (wrtBuf, "%s<TDS%s; TDS%s; TDSJDG %s; TDS%s; TDSPKMKY %fENT; TDSMULTI %fENT;TDS%s",
        	DetArr[detector], 
        	LvlUnitArr[displayUnit],
        	switchArr[judgement], 
        	resultArr[(ViInt16)result],
        	peakMKRYDelta, multiplier,preselArr[preselecter]);

    	total += NumFmtdBytes();
    }
               
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;    	    
}

/*===========================================================================*/
/* Function: Gated Spectrum						                             */
/* Purpose:  This function is used for Freq domain wave form analyzing. This */
/*           function sets the Gate Setup parameters.	     				 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt65_confGateTrg (ViSession instrumentHandle,
									   ViBoolean gateSource, ViBoolean settoSTD,
									   ViBoolean measMode, ViReal64 gatePosition,
									   ViReal64 gateWidth, ViInt16 trgSource, 
                                       ViInt16 triggerPosition, ViReal64 delayTime, 
                                       ViBoolean trgSlope, ViInt16 triggerLevel,
                                       ViBoolean gatedSweep, ViInt16 detector)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if (avopt65_invalidViBooleanRange (gateSource))
        return VI_ERROR_PARAMETER2;
    if (avopt65_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER3;
    if (avopt65_invalidViBooleanRange (measMode))
        return VI_ERROR_PARAMETER4;
    if (avopt65_invalidViReal64Range (gatePosition, 0, 1))
        return VI_ERROR_PARAMETER5;
    if (avopt65_invalidViReal64Range (gateWidth, 0, 1))
        return VI_ERROR_PARAMETER6;
	if (avopt65_invalidViInt16Range (trgSource,0,3))
        return VI_ERROR_PARAMETER7;
    if (avopt65_invalidViInt16Range (triggerPosition, 0, 100))
        return VI_ERROR_PARAMETER8;
    if (avopt65_invalidViReal64Range (delayTime, -1000.0, 1000.0))
        return VI_ERROR_INSTR_PARAMETER9;
    if (avopt65_invalidViBooleanRange (trgSlope))
        return VI_ERROR_INSTR_PARAMETER10;
    if (avopt65_invalidViInt16Range (triggerLevel, 0, 100))
        return VI_ERROR_INSTR_PARAMETER11;
    if (avopt65_invalidViBooleanRange (gatedSweep))
        return VI_ERROR_INSTR_PARAMETER12;    
    if (avopt65_invalidViInt16Range (detector, 0, 3))
        return VI_ERROR_INSTR_PARAMETER13;

   	Fmt (wrtBuf, "%s<TGTSRC TRG; TGTTRG %s; TGT%s; TGTTRGLVL %d; TGTTRGPOS %d; TGTTRGDT %f[e]SC",
    	 trgFdpSrcArr[trgSource], trgSlopeArr[trgSlope], triggerLevel, triggerPosition, delayTime);
    total += NumFmtdBytes();
	
   	if ( settoSTD == 1)          /* Set to STD: On */
    	Fmt (wrtBuf, "%s[a]<; TGTSETUP OFF; %s", gateExSTDArr[measMode]);
   		
   	else						 /* Set to STD: Off */
   		Fmt (wrtBuf, "%s[a]<; TGTPOS %f[e]SC; TGTWID %f[e]SC", gatePosition,gateWidth);
   	total += NumFmtdBytes();
   	
	if (gateSource == 0)      /* Gate Source: EXT */
	{
		Fmt (wrtBuf, "%s[a]<; TGTSRC EXT");
		total += NumFmtdBytes();
	}	

   	Fmt (wrtBuf, "%s[a]<; TGTSWP %s; TGT%s; TGTSETUP OFF", switchArr[gatedSweep],DetArr[detector]);
    total += NumFmtdBytes();
    
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;    
}

/*===========================================================================*/
/* Function: FDP Window Set						                             */
/* Purpose:  This function is used for Freq domain wave form analyzing. This */
/*           function sets the Gate Sweep Window.		     				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_window_setFDP (ViSession instrumentHandle,
                                        ViBoolean settoSTD, ViBoolean window,
                                        ViReal64 positionHz, ViReal64 widthHz)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;
    
    if (avopt65_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<FDPSETSTD");	
	}
	else
	{
	    if (avopt65_invalidViBooleanRange (window))
        	return VI_ERROR_PARAMETER3;
	    if (avopt65_invalidViReal64Range (positionHz, 0.0, 8.0e+9))
        	return VI_ERROR_PARAMETER4;
    	if (avopt65_invalidViReal64Range (widthHz, 0.0, 8.0e+9))
        	return VI_ERROR_PARAMETER5;
        	
		Fmt (wrtBuf, "%s<FDPWDO %s;FDPWPOS %fHZ; FDPWWID %fHZ", 
			switchArr[window], positionHz, widthHz);
	}
        
        
    total += NumFmtdBytes();
    
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;

}

/*===========================================================================*/
/* Function: FDP Parameter Set					                             */
/* Purpose:  This function is used for Freq domain wave form analyzing. This */
/*           function sets the Gate Sweep Window.		     				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_parameter_setFDP (ViSession instrumentHandle,
                                           ViBoolean settoSTD, ViInt16 detector,
                                           ViInt16 displayUnit,
                                           ViBoolean gatedSweep,
                                           ViBoolean judgement,
                                           ViReal64 upperLimit,
                                           ViReal64 lowerLimit)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;
    
    if (avopt65_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<FDPSETSTD");	
	    total += NumFmtdBytes();
	}
	else
	{
    	if (avopt65_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER3;
    	if (avopt65_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER4;
		if (avopt65_invalidViBooleanRange (gatedSweep))
        	return VI_ERROR_PARAMETER5;     	
   		if (avopt65_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER6;        

        Fmt (wrtBuf, "%s<FDP%s; FDP%s;TGTSWP %s; FDPJDG %s; FDPJDGUP %f; FDPJDGLOW %f",
        	DetArr[detector], LvlUnitArr[displayUnit],
        	switchArr[gatedSweep], switchArr[judgement],
        	upperLimit,  lowerLimit);
    	total += NumFmtdBytes();	
	}
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;	
}

/*===========================================================================*/
/* Function: OBW Parameter Set                                               */
/* Purpose:  This function sets the parameters for OBW measurement.			 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_parameter_setOBW (ViSession instrumentHandle,
                                           ViBoolean settoSTD, ViReal64 OBW,
                                           ViInt16 detector, ViBoolean judgement,
                                           ViReal64 upperLimit,
                                           ViReal64 lowerLimit)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;
    
    if (avopt65_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<OBWSETSTD");	
	    total += NumFmtdBytes();    
	
	}
	else
	{
	    if (avopt65_invalidViReal64Range (OBW, 0.5, 99.5))
        	return VI_ERROR_PARAMETER3;
    	if (avopt65_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER4;
   		if (avopt65_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER5;        
	    if (avopt65_invalidViReal64Range (upperLimit, 0, 100e+6))
        	return VI_ERROR_PARAMETER6;
	    if (avopt65_invalidViReal64Range (lowerLimit, 0, 100e+6))
        	return VI_ERROR_PARAMETER7;

        Fmt (wrtBuf, "%s<OBWPER %f; OBW%s; OBWJDG %s; OBWJDGUP %f; OBWJDGLOW %f",
        	OBW, DetArr[detector],
			switchArr[judgement], upperLimit, lowerLimit);
    	total += NumFmtdBytes();	
	}
        
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;		
}


/*===========================================================================*/
/* Function: DT Parameter Set                                                */
/* Purpose:  This function sets the parameters for Due to Transient			 */
/*	measurement.			                                                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_parameter_setDT (ViSession instrumentHandle,
                                          ViBoolean settoSTD, ViInt16 detector,
                                          ViInt16 displayUnit,
                                          ViBoolean TMPLCoupleofPWR,
                                          ViReal64 templateLimit,
                                          ViBoolean judgement,
                                          ViBoolean freqSetting, ViInt16 result,
                                          ViBoolean refPower,
                                          ViReal64 symbolRate, ViReal64 rolloff)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;
    
    if (avopt65_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<DTSSETSTD");	
	    total += NumFmtdBytes();

	}
	else
	{
    	if (avopt65_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER3;
   		if (avopt65_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER4;
   		if (avopt65_invalidViBooleanRange (TMPLCoupleofPWR))
        	return VI_ERROR_PARAMETER5; 
   		if (avopt65_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER7;  
   		if (avopt65_invalidViBooleanRange (freqSetting))
        	return VI_ERROR_PARAMETER8;         	
    		if (avopt65_invalidViInt16Range (result, 0, 2))
        	return VI_ERROR_INSTR_PARAMETER9;
   		if (avopt65_invalidViBooleanRange (refPower))
        	return VI_ERROR_INSTR_PARAMETER10;
   		if (avopt65_invalidViReal64Range (symbolRate, 1.0, 1.0e+9))
        	return VI_ERROR_INSTR_PARAMETER11;
   		if (avopt65_invalidViReal64Range (rolloff, 0.01, 0.99))
        	return VI_ERROR_INSTR_PARAMETER12;
        
        Fmt (wrtBuf, "%s<DTS%s; DTS%s; DTS%s; DTS%s; DTS%s;DTSTMPLPW %s;DTSTMPLBTM %fDB; DTSJDG %s; DTSSYMRT %fHZ; DTSRFACT %f",
        	FreqSetArr[freqSetting], DetArr[detector], dtsResultArr[result],
        	 refMkrArr[refPower], LvlUnitArr[displayUnit],switchArr[TMPLCoupleofPWR],
			templateLimit, switchArr[judgement], symbolRate, rolloff);
    	total += NumFmtdBytes();	
	}
        
    
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;	    
}

/*===========================================================================*/
/* Function: DT Template Set                                                 */
/* Purpose:  This function sets the parameters for Due to Transient			 */
/*	measurement.			                                                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_template_setDT (ViSession instrumentHandle,
                                         ViBoolean template, ViReal64 shiftXHz,
                                         ViReal64 shiftYdB, ViReal64 deltaXHz)
{
  	ViUInt32 i,retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;    

    if (avopt65_invalidViBooleanRange (template))
       	return VI_ERROR_PARAMETER2;
    if (avopt65_invalidViReal64Range (shiftXHz, -8.0e+9, 8.0e+9))
       	return VI_ERROR_PARAMETER3;   
    if (avopt65_invalidViReal64Range (shiftYdB, -200.0, 100))
       	return VI_ERROR_PARAMETER4; 
	if (avopt65_invalidViReal64Range (deltaXHz, -100e+6, 100e+6 ))
       	return VI_ERROR_PARAMETER5; 

    Fmt (wrtBuf, "%s<DTSTMPL %s; DTSTMPLSX %fHZ; DTSTMPLSY %fDB; DTSTMPLDX %fHZ",
    	switchArr[template],
    	shiftXHz, shiftYdB,deltaXHz);
 
    total += NumFmtdBytes();
	if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;       
}



/*===========================================================================*/
/* Function: DT Template Set                                                 */
/* Purpose:  This function sets the parameters for Due to Transient			 */
/*	measurement.			                                                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_template_editDT (ViSession instrumentHandle,
                                          ViInt16 operation,
                                          ViReal64 templateFreqHz,
                                          ViReal64 templateLeveldB)
{
  	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;   

    if (avopt65_invalidViInt16Range (operation, 0, 2))
        return VI_ERROR_PARAMETER2;

	switch ( operation )
	{
	case 0: /* Copy */
		Fmt (wrtBuf, "%s<DTSTMPLCP");	
		break;
	case 1: /* entry */
		if (avopt65_invalidViReal64Range (templateFreqHz, -100.0e+9, 100.0e+9))
        	return VI_ERROR_PARAMETER3;
    	if (avopt65_invalidViReal64Range (templateLeveldB, -200, 100))
        	return VI_ERROR_PARAMETER4;
        Fmt (wrtBuf, "%s<DTSTMPLED %fHZ,%fDB",templateFreqHz,templateLeveldB);
		break;
	case 2: /* Init */
		Fmt (wrtBuf, "%s<DTSTMPLCLR");   
		break;
	}

    total += NumFmtdBytes();	
	
	if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;       	
	
}

/*===========================================================================*/
/* Function: DT Marker Edit                                                  */
/* Purpose:  This function sets the parameters for Due to Transient			 */
/*	measurement.			                                                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_marker_editDT (ViSession instrumentHandle,
                                        ViInt16 operation,
                                        ViInt16 refMarkerType,
                                        ViReal64 refBandWidthHz,
                                        ViBoolean offsetMkrType,
                                        ViReal64 offsetFreqHz,
                                        ViReal64 bandWidthHz,
                                        ViReal64 limitPowerdB)
{
  	ViUInt32 i, retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;    

	if (avopt65_invalidViInt16Range (operation, 0, 3))
        return VI_ERROR_PARAMETER2;

	switch (operation)
	{
		case 0: /*  STD */
			Fmt (wrtBuf, "%s<DTSMKRCP");	
			break;
			
		case 1: /* Entry */
			if (avopt65_invalidViBooleanRange (offsetMkrType))
        		return VI_ERROR_PARAMETER5;
    		if (avopt65_invalidViReal64Range(offsetFreqHz, 0, 100.0e+9))
        		return VI_ERROR_PARAMETER6;
    		if (avopt65_invalidViReal64Range(bandWidthHz, 0, 100.0e+9))
        		return VI_ERROR_PARAMETER7;
    		if (avopt65_invalidViReal64Range(limitPowerdB, -200, 100))
        		return VI_ERROR_PARAMETER8;			
			
			
           	Fmt (wrtBuf, "%s<DTSMKRED %d,%fHZ,%fHZ,%fDB",offsetMkrType,offsetFreqHz,bandWidthHz,limitPowerdB);
			
			break;
		case 2:	/* Init */
			Fmt (wrtBuf, "%s<DTSMKRCLR");
			break;
		case 3: /* Ref */
			if (avopt65_invalidViInt16Range (refMarkerType, 0, 2))
        		return VI_ERROR_PARAMETER3;
    		if (avopt65_invalidViReal64Range (refBandWidthHz, 0, 1.0e+9))
        		return VI_ERROR_PARAMETER4;           	

			Fmt (wrtBuf, "%s<DTSMKRCLR; DTSMKRED %d,0HZ,%fHZ,0DB",refMarkerType,refBandWidthHz);

			break;
	}
	total += NumFmtdBytes();
	
	if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;      
}

/*===========================================================================*/
/* Function: DM Parameter Set                                                */
/* Purpose:  This function sets the parameters for Due to modulation		 */
/*	measurement.			                                                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_parameter_setDM (ViSession instrumentHandle,
                                          ViBoolean settoSTD,
                                          ViBoolean freqSetting,
                                          ViInt16 detector, ViInt16 result,
                                          ViBoolean refPower,
                                          ViInt16 displayUnit,
                                          ViBoolean TMPLCoupleofPWR,
                                          ViBoolean judgement,
                                          ViReal64 templateLimit,
                                          ViReal64 symbolRate, ViReal64 rolloff)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;
    
    if (avopt65_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<DTMSETSTD");	
	   total += NumFmtdBytes();            
	}
	else
	{
   		if (avopt65_invalidViBooleanRange (freqSetting))
        	return VI_ERROR_PARAMETER3;       
    	if (avopt65_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER4;
    	if (avopt65_invalidViInt16Range (result, 0, 2))
        	return VI_ERROR_PARAMETER5;
  		if (avopt65_invalidViBooleanRange (refPower))
        	return VI_ERROR_PARAMETER6;           	
   		if (avopt65_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER7;
   		if (avopt65_invalidViBooleanRange (TMPLCoupleofPWR))
        	return VI_ERROR_PARAMETER8; 
   		if (avopt65_invalidViBooleanRange (judgement))
        	return VI_ERROR_INSTR_PARAMETER9;  
   		if (avopt65_invalidViReal64Range (symbolRate, 1.0, 1.0e+9))
        	return VI_ERROR_INSTR_PARAMETER11;  
   		if (avopt65_invalidViReal64Range (rolloff, 0.01, 0.99))
        	return VI_ERROR_INSTR_PARAMETER11;  
  	
        Fmt (wrtBuf, "%s<DTM%s; DTM%s; DTM%s; DTM%s; DTM%s; DTMTMPLPW %s; DTMTMPLBTM %fDB; DTMJDG %s; DTMSYMRT %fHZ; DTMRFACT %f",
        	FreqSetArr[freqSetting], DetArr[detector], dtsResultArr[result],
        	refMkrArr[refPower], LvlUnitArr[displayUnit],switchArr[TMPLCoupleofPWR],
			templateLimit, switchArr[judgement], symbolRate, rolloff);
    	total += NumFmtdBytes();	
	}
        
 
    
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;	    	    
}

/*===========================================================================*/
/* Function: DM Template Set                                                 */
/* Purpose:  This function sets the parameters for Due to modulation		 */
/*	measurement. This function sets for Template.			                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_template_setDM (ViSession instrumentHandle,
                                         ViBoolean template, ViReal64 shiftXHz,
                                         ViReal64 shiftYdB, ViReal64 deltaXHz)
{
  	ViUInt32 i,retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;    

    if (avopt65_invalidViBooleanRange (template))
       	return VI_ERROR_PARAMETER2;
    if (avopt65_invalidViReal64Range (shiftXHz, -8.0e+9, 8.0e+9))
       	return VI_ERROR_PARAMETER3;   
    if (avopt65_invalidViReal64Range (shiftYdB, -200.0, 100))
       	return VI_ERROR_PARAMETER4; 
	if (avopt65_invalidViReal64Range (deltaXHz, -100e+6, 100e+6 ))
       	return VI_ERROR_PARAMETER5; 

    Fmt (wrtBuf, "%s<DTMTMPL %s; DTMTMPLSX %fHZ; DTMTMPLSY %fDB; DTMTMPLDX %fHZ",
    	switchArr[template], shiftXHz, shiftYdB, deltaXHz);
 
    total += NumFmtdBytes();
	if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;           
}

/*===========================================================================*/
/* Function: DM Template Edit                                                */
/* Purpose:  This function edets the parameters for Due to modulation		 */
/*	measurement. This function sets for Template.			                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_template_editDM (ViSession instrumentHandle,
                                          ViInt16 operation,
                                          ViReal64 templateFreqHz,
                                          ViReal64 templateLevel)
{
   	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;   

    if (avopt65_invalidViInt16Range (operation, 0, 2))
        return VI_ERROR_PARAMETER2;

	switch ( operation )
	{
	case 0: /* Copy */
		Fmt (wrtBuf, "%s<DTMTMPLCP");	
		break;
	case 1: /* entry */
		if (avopt65_invalidViReal64Range (templateFreqHz, -100.0e+9, 100.0e+9))
        	return VI_ERROR_PARAMETER3;
        Fmt (wrtBuf, "%s<DTMTMPLED %fHZ,%f[e]",templateFreqHz,templateLevel);
		break;
	case 2: /* Init */
		Fmt (wrtBuf, "%s<DTMTMPLCLR");   
		break;
	}

    total += NumFmtdBytes();	
	
	if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;   
}

/*===========================================================================*/
/* Function: DM Marker Table Edit  											 */
/* Purpose:  This function sets the parameters for Due to modulation		 */
/*	measurement. This function sets for Template.			                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_marker_editDM (ViSession instrumentHandle,
                                        ViInt16 operation,
                                        ViInt16 refMarkerType,
                                        ViReal64 refBandWidthHz,
                                        ViBoolean offsetMkrType,
                                        ViReal64 offsetFreqHz,
                                        ViReal64 bandWidthHz,
                                        ViReal64 limitPowerdB)
{
  	ViUInt32 i, retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;    

	if (avopt65_invalidViInt16Range (operation, 0, 3))
        return VI_ERROR_PARAMETER2;

	switch (operation)
	{
		case 0: /*  STD */
			Fmt (wrtBuf, "%s<DTMMKRCP");	
			break;
			
		case 1: /* Entry */
			if (avopt65_invalidViBooleanRange (offsetMkrType))
        		return VI_ERROR_PARAMETER5;
    		if (avopt65_invalidViReal64Range(offsetFreqHz, 0, 100.0e+9))
        		return VI_ERROR_PARAMETER6;
    		if (avopt65_invalidViReal64Range(bandWidthHz, 0, 100.0e+9))
        		return VI_ERROR_PARAMETER7;
    		if (avopt65_invalidViReal64Range(limitPowerdB, -200, 100))
        		return VI_ERROR_PARAMETER8;			
			
			Fmt (wrtBuf, "%s<DTMMKRED %d,%fHZ,%fHZ,%fDB",offsetMkrType,offsetFreqHz,bandWidthHz,limitPowerdB);

			break;
		case 2:	/* Init */
			Fmt (wrtBuf, "%s<DTMMKRCLR");
			break;
		case 3: /* Ref */
			if (avopt65_invalidViInt16Range (refMarkerType, 0, 2))
        		return VI_ERROR_PARAMETER3;
    		if (avopt65_invalidViReal64Range (refBandWidthHz, 0, 1.0e+9))
        		return VI_ERROR_PARAMETER4; 
			Fmt (wrtBuf, "%s<DTMMKRCLR; DTMMKRED %d,0HZ,%fHZ,0DB",refMarkerType,refBandWidthHz);
			break;
	}
	total += NumFmtdBytes();
	if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;          
}


/*===========================================================================*/
/* Function: InBand Spurious Parameter Set                                   */
/* Purpose:  This function sets the parameters for In Band Spurious			 */
/*           measurement.													 */
/*===========================================================================*/


ViStatus CVIFUNC avopt65_parameter_setINSPR (ViSession instrumentHandle,
                                             ViBoolean settoSTD,
                                             ViBoolean freqSetting,
                                             ViInt16 detector,
                                             ViBoolean refPower,
                                             ViReal64 peakMkrYDelta,
                                             ViInt16 result, ViInt16 displayUnit,
                                             ViBoolean TMPLCoupleofPWR,
                                             ViReal64 templateLimit,
                                             ViBoolean judgement)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;
    
    if (avopt65_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<SPRSETSTD");	
	}
	else
	{
   		if (avopt65_invalidViBooleanRange (freqSetting))
        	return VI_ERROR_PARAMETER3;       
    	if (avopt65_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER4;
  		if (avopt65_invalidViBooleanRange (refPower))
        	return VI_ERROR_PARAMETER5; 
    	if (avopt65_invalidViReal64Range (peakMkrYDelta, 0.1, 10.0))
        	return VI_ERROR_PARAMETER6;           	
   		if (avopt65_invalidViInt16Range (result, 0, 2))
        	return VI_ERROR_PARAMETER7;        	
   		if (avopt65_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER8;
   		if (avopt65_invalidViBooleanRange (TMPLCoupleofPWR))
        	return VI_ERROR_INSTR_PARAMETER9; 
   		if (avopt65_invalidViBooleanRange (judgement))
        	return VI_ERROR_INSTR_PARAMETER11; 

        Fmt (wrtBuf, "%s<SPR%s; SPR%s;SPRPKMKY %fENT; SPR%s; SPR%s; SPR%s;SPRTMPLPW %s;SPRTMPLBTM %f[e]; SPRJDG %s",
        	FreqSetArr[freqSetting], DetArr[detector],peakMkrYDelta, dtsResultArr[result],
        	refMkrArr[refPower], LvlUnitArr[displayUnit],switchArr[TMPLCoupleofPWR],
			templateLimit, switchArr[judgement]);
	}
        
    total += NumFmtdBytes();
    
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;	    	        
}

/*===========================================================================*/
/* Function: InBand Spurious Template Set                                    */
/* Purpose:  This function sets the parameters for In Band Spurious			 */
/*           measurement.													 */
/*===========================================================================*/

ViStatus CVIFUNC avopt65_template_setINSPR (ViSession instrumentHandle,
                                            ViBoolean template,
                                            ViReal64 shiftXHz, ViReal64 shiftYdB,
                                            ViReal64 deltaXHz)
{
  	ViUInt32 i, retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE],wrtBuf2[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;    


    if (avopt65_invalidViBooleanRange (template))
       	return VI_ERROR_PARAMETER2;
    if (avopt65_invalidViReal64Range (shiftXHz, -8.0e+9, 8.0e+9))
       	return VI_ERROR_PARAMETER3;   
    if (avopt65_invalidViReal64Range (shiftYdB, -200.0, 100))
       	return VI_ERROR_PARAMETER4; 
	if (avopt65_invalidViReal64Range (deltaXHz, -100e+6, 100e+6))
       	return VI_ERROR_PARAMETER5; 

    Fmt (wrtBuf, "%s<SPRTMPL %s; SPRTMPLSX %fHZ; SPRTMPLSY %fDB; SPRTMPLDX %fHZ",switchArr[template],
    	shiftXHz, shiftYdB,deltaXHz);
 
    total += NumFmtdBytes();
	if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;	        
}

/*===========================================================================*/
/* Function: InBand Spurious Template Edit                                   */
/* Purpose:  This function sets the parameters for In Band Spurious			 */
/*           measurement.													 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_template_editINSPR (ViSession instrumentHandle,
                                             ViInt16 operation,
                                             ViReal64 templateFreqHz,
                                             ViReal64 templateLevel)
{
   	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;   

    if (avopt65_invalidViInt16Range (operation, 0, 2))
        return VI_ERROR_PARAMETER2;

	switch ( operation )
	{
	case 0: /* Copy */
		Fmt (wrtBuf, "%s<SPRTMPLCP");	
		break;
	case 1: /* entry */
		if (avopt65_invalidViReal64Range (templateFreqHz, -100.0e+9, 100.0e+9))
        	return VI_ERROR_PARAMETER3;
        Fmt (wrtBuf, "%s<SPRTMPLED %fHZ,%f[e]",templateFreqHz,templateLevel);
		break;
	case 2: /* Init */
		Fmt (wrtBuf, "%s<SPRTMPLCLR");   
		break;
	}

    total += NumFmtdBytes();	
	
	if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;   
}    


/*===========================================================================*/
/* Function: InBand Spurious Marker Set                                      */
/* Purpose:  This function sets the parameters for In Band Spurious			 */
/*           measurement.													 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_marker_editINSPR (ViSession instrumentHandle,
                                           ViInt16 operation,
                                           ViBoolean refMarkerType,
                                           ViReal64 refBandWidthHz,
                                           ViReal64 startFreqHz,
                                           ViReal64 stopFreqHz,
                                           ViReal64 limitPower)
{
 	ViUInt32 i, retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;    

	if (avopt65_invalidViInt16Range (operation, 0, 3))
        return VI_ERROR_PARAMETER2;

	switch (operation)
	{
		case 0: /*  STD */
			Fmt (wrtBuf, "%s<SPRMKRCP");	
			break;
			
		case 1: /* Entry */
    			if (avopt65_invalidViReal64Range(startFreqHz, 0, 100.0e+9))
        		return VI_ERROR_PARAMETER5;
    			if (avopt65_invalidViReal64Range(stopFreqHz, 0, 100.0e+9))
        		return VI_ERROR_PARAMETER6;

            	Fmt (wrtBuf, "%s<SPRMKRED 0,%fHZ,%fHZ,%f[e]", startFreqHz, stopFreqHz, limitPower);
			break;
		case 2:	/* Init */
			Fmt (wrtBuf, "%s<SPRMKRCLR");
			break;
		case 3: /* Ref */
			if (avopt65_invalidViBooleanRange (refMarkerType))
        		return VI_ERROR_PARAMETER3;
    		if (avopt65_invalidViReal64Range (refBandWidthHz, 0, 1.0e+9))
        		return VI_ERROR_PARAMETER4;     
			Fmt (wrtBuf, "%s<SPRMKRCLR; SPRMKRED %d,0HZ,%fHZ,0DB",refMarkerType,refBandWidthHz);
			break;
	}
	
	total += NumFmtdBytes();
	
	if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;    
}

/*===========================================================================*/
/* Function: OutBand Spurious Parameter Set                                  */
/* Purpose:  This function sets the parameters for In Band Spurious			 */
/*           measurement.													 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_parameter_setOUTSPR (ViSession instrumentHandle,
                                              ViBoolean settoSTD,
                                              ViInt16 detector,
                                              ViReal64 peakMkrYDelta,
                                              ViInt16 displayUnit,
                                              ViBoolean judgement,
                                              ViBoolean preselector)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;
    
    if (avopt65_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<FDSSETSTD");	
	}
	else
	{
	
    	if (avopt65_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER3;
    	if (avopt65_invalidViReal64Range (peakMkrYDelta, 0.1, 10.0))
        	return VI_ERROR_PARAMETER4;           	
   		if (avopt65_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER5;
   		if (avopt65_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER6; 
   		if (avopt65_invalidViBooleanRange (preselector))
        	return VI_ERROR_PARAMETER7; 

        Fmt (wrtBuf, "%s<FDS%s;FDSPKMKY %fENT; FDS%s; FDSJDG %s; FDS%s",
        	DetArr[detector], peakMkrYDelta,
        	LvlUnitArr[displayUnit], switchArr[judgement],preselArr[preselector]);
	}
        
    total += NumFmtdBytes();
    
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;    
}

/*===========================================================================*/
/* Function: OutBand Spurious Table Set                                      */
/* Purpose:  This function sets the parameters for In Band Spurious			 */
/*           measurement.													 */
/*===========================================================================*/

ViStatus CVIFUNC avopt65_table_editOUTSPR (ViSession instrumentHandle,
                                           ViInt16 operation,
                                           ViInt16 tableNumber,
                                           ViReal64 startFreqHz,
                                           ViReal64 stopFreqHz, ViReal64 RBW,
                                           ViReal64 VBW, ViReal64 sweepTime,
                                           ViReal64 limitLevel)
{
 	ViUInt32 total = 0;       
    ViUInt32 i, retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

	
	if (avopt65_invalidViInt16Range (operation, 0, 4))
        return VI_ERROR_PARAMETER2;
	if (avopt65_invalidViInt16Range (tableNumber, 1, 3))
        return VI_ERROR_PARAMETER3;

    switch (operation)
    {
    	case 0: /* Load */
    		Fmt (wrtBuf, "%s<FDSTBL %d; FDSLD",tableNumber);
    		break;
    	case 1: /* Save */
    		Fmt (wrtBuf, "%s<FDSTBL %d; FDSSV",tableNumber);
    		break;
    	case 2: /* Sel */
    		Fmt (wrtBuf, "%s<FDSTBL %d",tableNumber);
    		break;
    	case 3: /* Edit */
    		if (avopt65_invalidViReal64Range (startFreqHz, -200e+6, 8e+9))
        		return VI_ERROR_PARAMETER4;
			if (avopt65_invalidViReal64Range (stopFreqHz, 0.0, 8.2e+9))
        		return VI_ERROR_PARAMETER5;
			if (avopt65_invalidViReal64Range (RBW, 10.0, 10e+6))
        		return VI_ERROR_PARAMETER6;
        	if (avopt65_invalidViReal64Range (VBW, 1.0, 10e+6))
        		return VI_ERROR_PARAMETER7;
        	if (avopt65_invalidViReal64Range (sweepTime, 1e-6, 1000))
        		return VI_ERROR_PARAMETER8;        	

     		Fmt (wrtBuf, "%s<FDSTBLED %fHZ,%fHZ, %fHZ, %fHZ, %f[e]SC,%f[e]",
     			startFreqHz,stopFreqHz,RBW,VBW,sweepTime,limitLevel);
    		break;
    	case 4: /* Init */
    		Fmt (wrtBuf, "%s<FDSCLR");
    	
    		break;
    }
   total += NumFmtdBytes();
	
	if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;    
}

/*===========================================================================*/
/* Function: CDP Coefficient Parameter Set	                                 */
/* Purpose:  This function sets the parameters for CDP coefficient			 */
/*           measurement.													 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt65_parameter_setCDPCoef (ViSession instrumentHandle,
                                                ViInt32 msRng, ViReal64 tauOffset,
                                                ViBoolean cmpFltr, ViReal64 rolloff,
                                                ViBoolean eqlFltr, ViBoolean PNOfstSrch,
                                                ViInt16 PNOfst, ViInt16 crrFreqSrch,
                                                ViInt16 trgSrc, ViBoolean extTrgSlp,
                                                ViInt16 threshold, ViBoolean chnlDef,
                                                ViInt16 WalshCdLen, ViInt16 QOF,
                                                ViBoolean Paley)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if (avopt65_invalidViInt32Range (msRng, 128, 36864))
        return VI_ERROR_PARAMETER2;
    if (avopt65_invalidViReal64Range (tauOffset, -500.0e-6, 500.0e-6))
        return VI_ERROR_PARAMETER3;
    if (avopt65_invalidViBooleanRange (cmpFltr))
        return VI_ERROR_PARAMETER4;
    if (avopt65_invalidViReal64Range (rolloff, 0.05, 0.2))
    	return VI_ERROR_PARAMETER5;
    if (avopt65_invalidViBooleanRange (eqlFltr))
        return VI_ERROR_PARAMETER6;
    if (avopt65_invalidViBooleanRange (PNOfstSrch))
        return VI_ERROR_PARAMETER7;
    if (avopt65_invalidViInt16Range (PNOfst, 0, 511))
        return VI_ERROR_PARAMETER8;
    if (avopt65_invalidViInt16Range (crrFreqSrch, 0, 2))
        return VI_ERROR_INSTR_PARAMETER9;
    if (avopt65_invalidViInt16Range (trgSrc, 0, 3))
        return VI_ERROR_INSTR_PARAMETER10;
    if (avopt65_invalidViBooleanRange (extTrgSlp))
        return VI_ERROR_INSTR_PARAMETER11;
    if (avopt65_invalidViInt16Range (threshold, -50, 0))
        return VI_ERROR_INSTR_PARAMETER12;
    if (avopt65_invalidViBooleanRange (chnlDef))
        return VI_ERROR_INSTR_PARAMETER13;
    if (avopt65_invalidViInt16Range (WalshCdLen, 4, 128))
        return VI_ERROR_INSTR_PARAMETER14;
    if (avopt65_invalidViInt16Range (QOF, 0, 3))
        return VI_ERROR_INSTR_PARAMETER15;
    if (avopt65_invalidViBooleanRange (Paley))
        return VI_ERROR_INSTR_PARAMETER16;

	Fmt (wrtBuf, "%s<C2PCMRNG %d; C2TOFS %f[e]; C2CMPFLT %s; C2RFACT %f; C2EQFLT %s; C2PNMOD %s; C2PNOFS %d; %s; C2%s; C2%s; C2THRSH %d; C2CHDEF %s; C2WLEN %d; C2QOF %d; C2PALEY %s",
		msRng, tauOffset, switchArr[cmpFltr], rolloff, switchArr[eqlFltr],
		switchArr[PNOfstSrch], PNOfst, crrFreqSrchArr[crrFreqSrch],
		trgModeArr[trgSrc], trgSlopeArr[extTrgSlp], threshold, 
		switchArr[chnlDef], WalshCdLen, QOF, switchArr[Paley]);

    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;    
}

/*===========================================================================*/
/* Function: CDP Coefficient Scale Set		                                 */
/* Purpose:  This function sets the scale for CDP coefficient measurement.	 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt65_scale_setCDPCoef (ViSession instrumentHandle,
                                            ViInt16 format, ViBoolean display,
                                            ViInt16 yScale, ViInt16 ySclUpper,
                                            ViBoolean yDiv, ViInt16 tblPage)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if (avopt65_invalidViInt16Range (format, 0, 2))
        return VI_ERROR_PARAMETER2;
    if (avopt65_invalidViBooleanRange (display))
        return VI_ERROR_PARAMETER3;
    if (avopt65_invalidViInt16Range (yScale, 0, 4))
        return VI_ERROR_PARAMETER4;
    if (avopt65_invalidViInt16Range (ySclUpper, -50, 70))
        return VI_ERROR_PARAMETER5;
    if (avopt65_invalidViBooleanRange (yDiv))
        return VI_ERROR_PARAMETER6;
    if (avopt65_invalidViInt16Range (tblPage, 1, 2))
        return VI_ERROR_PARAMETER7;
 
    Fmt (wrtBuf, "%s<%s; %s;C2PCYSCL %s; C2YUPR %dDB; %s; C2PCPAGE %d",
    	sclFmtArr[format], sclDispArr[display], sclYCDPCFArr[yScale],
    	ySclUpper, yDivArr[yDiv], tblPage);
    	
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;    
}

/*===========================================================================*/
/* Function: CDP Coefficient Channel Define Table Edit                       */
/* Purpose:  This function edit the channel define table.					 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt65_chDefTbl_editCDPCoef (ViSession instrumentHandle,
                                                ViInt16 ttlChnlN, ViInt16 editChnl,
                                                ViInt16 chnlName, ViInt16 radioCnfg,
                                                ViInt32 dataRate, ViInt16 valQOF,
                                                ViInt16 walshCdN)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if (avopt65_invalidViInt16Range (ttlChnlN, 1, 128))
        return VI_ERROR_PARAMETER2;
    if (avopt65_invalidViInt16Range (editChnl, 0, 127))
        return VI_ERROR_PARAMETER3;
    if (avopt65_invalidViInt16Range (chnlName, 0, 8))
        return VI_ERROR_PARAMETER4;
    if (avopt65_invalidViInt16Range (radioCnfg, 1, 5))
        return VI_ERROR_PARAMETER5;
    if (avopt65_invalidViInt32Range (dataRate, 1200, 307200))
        return VI_ERROR_PARAMETER6;
    if (avopt65_invalidViInt16Range (valQOF, 0, 3))
        return VI_ERROR_PARAMETER7;
    if (avopt65_invalidViInt16Range (walshCdN, 0, 127))
        return VI_ERROR_PARAMETER8;
        
    Fmt (wrtBuf, "%s<C2CHTOTAL %d; C2CHEDIT %d; %s; C2CHRC %d, C2CHRATE %d; C2CHQOF %d; C2CHWNUM %d",
    	ttlChnlN, editChnl, chnlNameArr[chnlName], radioCnfg, dataRate,
    	valQOF, walshCdN);
    	
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;    
}

/*===========================================================================*/
/* Function: CDP Coefficient Options Set	                                 */
/* Purpose:  This function sets the options for CDP coefficient measurement. */
/*===========================================================================*/
ViStatus _VI_FUNC avopt65_measOption_setCDPCoef (ViSession instrumentHandle,
                                                 ViInt16 grphPltTyp, ViInt16 pwrUnit,
                                                 ViReal64 pltChnlPwr, ViBoolean dltTau,
                                                 ViBoolean dltTheta, ViBoolean cdDmErr,
                                                 ViBoolean sigPwr, ViBoolean swEVM,
                                                 ViBoolean fxSymblLvl)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if (avopt65_invalidViInt16Range (grphPltTyp, 0, 2))
        return VI_ERROR_PARAMETER2;
    if (avopt65_invalidViInt16Range (pwrUnit, 0, 2))
        return VI_ERROR_PARAMETER3;
    if (avopt65_invalidViReal64Range (pltChnlPwr, -50.0, 50.0))
        return VI_ERROR_PARAMETER4;
    if (avopt65_invalidViBooleanRange (dltTau))
        return VI_ERROR_PARAMETER5;
    if (avopt65_invalidViBooleanRange (dltTheta))
        return VI_ERROR_PARAMETER6;
    if (avopt65_invalidViBooleanRange (cdDmErr))
        return VI_ERROR_PARAMETER7;
    if (avopt65_invalidViBooleanRange (sigPwr))
        return VI_ERROR_PARAMETER8;
    if (avopt65_invalidViBooleanRange (swEVM))
        return VI_ERROR_INSTR_PARAMETER9;
    if (avopt65_invalidViBooleanRange (fxSymblLvl))
        return VI_ERROR_INSTR_PARAMETER10;
        
    Fmt (wrtBuf, "%s<%s; %s; C2PIPWR %fDB; C2TAU %s; C2THETA %s; C2CDE %s; C2SIGPOW %s; C2EVM %s; C2FIXSYM %s",
    	grphPltTypArr[grphPltTyp], pwrUnitCDPArr[pwrUnit], pltChnlPwr,
    	switchArr[dltTau], switchArr[dltTheta], switchArr[cdDmErr],
    	switchArr[sigPwr], switchArr[swEVM], switchArr[fxSymblLvl]);
    
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;    
}

/*===========================================================================*/
/* Function: CDP Parameter Set	                                 			 */
/* Purpose:  This function sets the parameters for CDP measurement.			 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt65_parameter_setCDP (ViSession instrumentHandle,
                                            ViBoolean msMode, ViInt16 msRng, 
                                            ViInt16 threshold, ViBoolean pnDlySrch,
                                            ViInt16 pnDly, ViInt16 trgSrc,
                                            ViBoolean extTrgSlp, ViReal64 extTrgDly,
                                            ViBoolean freqMsRng)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;
    
    if (avopt65_invalidViBooleanRange (msMode))
        return VI_ERROR_PARAMETER2;
    if (avopt65_invalidViInt16Range (msRng, 128, 1536))
        return VI_ERROR_PARAMETER3;
    if (avopt65_invalidViInt16Range (threshold, -50, 0))
        return VI_ERROR_PARAMETER4;
    if (avopt65_invalidViBooleanRange (pnDlySrch))
        return VI_ERROR_PARAMETER5;
    if (avopt65_invalidViInt16Range (pnDly, 0, 511))
        return VI_ERROR_PARAMETER6;
    if (avopt65_invalidViInt16Range (trgSrc, 0, 3))
        return VI_ERROR_PARAMETER7;
    if (avopt65_invalidViBooleanRange (extTrgSlp))
        return VI_ERROR_PARAMETER8;
    if (avopt65_invalidViReal64Range (extTrgDly, -5.00e-3, 6.25e-3))
        return VI_ERROR_INSTR_PARAMETER9;
    if (avopt65_invalidViBooleanRange (freqMsRng))
        return VI_ERROR_INSTR_PARAMETER10;
        
    Fmt (wrtBuf, "%s<%s; C2MRNG %d; C2THRSH %dDB; C2PNMOD %s; C2PNDLY %d; C2%s; C2%s; C2TRGDLY %f[e]SC; %s",
    	msModeCDPArr[msMode], msRng, threshold, switchArr[pnDlySrch], pnDly,
    	trgModeArr[trgSrc], trgSlopeArr[extTrgSlp], extTrgDly, freqMsRngArr[freqMsRng]);
    	
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;    
}

/*===========================================================================*/
/* Function: CDP View Set	                                 			 	 */
/* Purpose:  This function sets the displaying contents on the R3267/73 's   */
/*           screen. 														 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt65_view_setCDP (ViSession instrumentHandle,
                                       ViInt16 display, ViInt16 viewChnl,
                                       ViBoolean yDiv)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if (avopt65_invalidViInt16Range (display, 0, 4))
        return VI_ERROR_PARAMETER2;
    if (avopt65_invalidViInt16Range (viewChnl, 0, 6))
        return VI_ERROR_PARAMETER3;
    if (avopt65_invalidViBooleanRange (yDiv))
        return VI_ERROR_PARAMETER4;
        
    Fmt (wrtBuf, "%s<%s; C2VWCH %s; %s",
        dispCDPArr[display], viewChnlArr[viewChnl], yDivArr[yDiv]);
        
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;    
}

/*===========================================================================*/
/* Function: CDP Channel Set                                 			 	 */
/* Purpose:  This function sets the measurement channels. 					 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt65_channel_setCDP (ViSession instrumentHandle,
                                          ViInt16 oprMode,
                                          ViBoolean sch1Walsh, ViInt16 sch1Rpt,
                                          ViBoolean sch2Walsh, ViInt16 sch2Rpt)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;
    
    if (avopt65_invalidViInt16Range (oprMode, 0, 2))
        return VI_ERROR_PARAMETER2;
    if (avopt65_invalidViBooleanRange (sch1Walsh))
        return VI_ERROR_PARAMETER3;
    if (avopt65_invalidViInt16Range (sch1Rpt, 1, 32))
        return VI_ERROR_PARAMETER4;
    if (avopt65_invalidViBooleanRange (sch2Walsh))
        return VI_ERROR_PARAMETER5;
    if (avopt65_invalidViInt16Range (sch2Rpt, 1, 16))
        return VI_ERROR_PARAMETER6;
    
    Fmt (wrtBuf, "%s<C2SCH2WALSH W68; %s; %s; C2SCH1REP %d; %s; C2SCH2REP %d",
    	oprModeArr[oprMode], sch1WalshArr[sch1Walsh], sch1Rpt, 
    	sch2WalshArr[sch2Walsh], sch2Rpt);
        
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;    
}

/*===========================================================================*/
/* Function: TX Power                                                        */
/* Purpose:  This function sets the parameters for TX Power measurement.	 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt65_parameter_setTxPower (ViSession instrumentHandle,
                                                ViInt16 trgMode, ViBoolean extTrgSlp,
                                                ViReal64 extTrgDly)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if (avopt65_invalidViInt16Range (trgMode, 0, 2))
        return VI_ERROR_PARAMETER2;
    if (avopt65_invalidViBooleanRange (extTrgSlp))
        return VI_ERROR_PARAMETER3;
    if (avopt65_invalidViReal64Range (extTrgDly, -10.0e-3, 10.0e-3))
        return VI_ERROR_PARAMETER5;
        
    Fmt (wrtBuf, "%s<TX%s; TX%s; TXTRGDLY %f[e]SC",
    	trgModeArr[trgMode], trgSlopeArr[extTrgSlp], extTrgDly);
    
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;    
}

/*===========================================================================*/
/* Function: Set CCDF measurement graph's scale.							 */
/* Purpose:  This function sets the scales for the graph of the CCDF	 	 */
/*			 measurement.													 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt65_scale_setCCDF (ViSession instrumentHandle,
                                         ViInt16 xSclMax, ViInt16 xSclRng,
                                         ViBoolean pwrUnit)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if (avopt65_invalidViInt16Range (xSclMax, -20, 70))
        return VI_ERROR_PARAMETER2;
    if (avopt65_invalidViInt16Range (xSclRng, 10, 50))
        return VI_ERROR_PARAMETER3;
    if (avopt65_invalidViBooleanRange (pwrUnit))
        return VI_ERROR_PARAMETER4;
        
    Fmt (wrtBuf, "C2CCDFXMAX %d; C2CCDFXRNG %d; C2CCDF%s", xSclMax, xSclRng, pwrUnitArr[pwrUnit]);
    
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;    
}

/*===========================================================================*/
/* Function: CCDF measurement parameters.									 */
/* Purpose:  This function sets the parameters of the CCDF measurement.		 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt65_parameter_setCCDF (ViSession instrumentHandle,
                                             ViBoolean trgMode, ViBoolean trgSlp,
                                             ViReal64 trgDly, ViInt32 msLength)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if (avopt65_invalidViBooleanRange (trgMode))
        return VI_ERROR_PARAMETER2;
    if (avopt65_invalidViBooleanRange (trgSlp))
        return VI_ERROR_PARAMETER3;
    if (avopt65_invalidViReal64Range (trgDly, -250.0e-6, 250.0e-6))
        return VI_ERROR_PARAMETER4;
    if (avopt65_invalidViInt32Range (msLength, 10000, 100000000))
        return VI_ERROR_PARAMETER5;
    
    Fmt (wrtBuf, "%s<%s; C2CCDF%s; C2CCDFTRGDLY %f[e]; C2CCDFMLEN %d",
    	trgMdCCDFArr[trgMode], trgSlopeArr[trgSlp], trgDly, msLength);
    	
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;    
}

/*===========================================================================*/
/* Function: Averaging                                                       */
/* Purpose:  This function sets the averaging times of Burst Envelope		 */
/*           Spectrum, Gated Output Power, On/Off Ratio, OBW,  Channel Power,*/
/*           In Band Spurious, Waveform Quality, Code Domain Power, TX Power */
/*           functions.														 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_average_set (ViSession instrSession, ViInt16 function,
				ViInt16 AVGTime)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if (avopt65_invalidViInt16Range (function, 0, 10))
        return VI_ERROR_PARAMETER2;
    if (avopt65_invalidViInt16Range (AVGTime, 1, 999))
        return VI_ERROR_PARAMETER3;

    Fmt (wrtBuf, "%s<%s %d[b2]", avgFunctionArr[function], AVGTime);

    if ((avopt65_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;
}

/*===========================================================================*/
/* Function: Y Scale Set	                                                 */
/* Purpose:  This function sets the vertical scale of the graph.			 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt65_Yscale_set (ViSession instrSession,
                                      ViInt16 function, ViInt16 scaleY)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if (avopt65_invalidViInt16Range (function, 0, 2))
        return VI_ERROR_PARAMETER2;
    if (avopt65_invalidViInt16Range (scaleY, 0, 2))
        return VI_ERROR_PARAMETER3;
    
    Fmt (wrtBuf, "%s<%s %s", yScaleFunctionArr[function], yScaleArr[scaleY]);    
    
    if ((avopt65_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;
}

/*===========================================================================*/
/* Function: Auto Level Set                                                  */
/* Purpose:  This function is used for Auto Level set in selected meas mode. */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_execute_auto_lvl (ViSession instrumentHandle,
                                           ViInt16 function)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if (avopt65_invalidViInt16Range (function, 0, 9))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", autoLvlFunctionArr[function]);

    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;    
}

/*===========================================================================*/
/* Function: CDP Coefficient Graph Type Select                               */
/* Purpose:  This function selects the graph type of the CDP coefficient.	 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt65_selectGraphTypeCDPCoef (ViSession instrumentHandle,
                                                  ViInt16 graphSlct, ViBoolean plotType)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if (avopt65_invalidViInt16Range (graphSlct, 0, 8))
        return VI_ERROR_PARAMETER2;
    if (avopt65_invalidViBooleanRange (plotType))
        return VI_ERROR_PARAMETER3;
    
    Fmt (wrtBuf, "%s<C2PC%s; C2PC%s", slctGraphArr[graphSlct], plotTypeArr[plotType]);    
    
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;    
}

/*===========================================================================*/
/* Function: CDP Graph Type Select				                             */
/* Purpose:  This function selects the graph type of the CDP.				 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt65_selectGraphTypeCDP (ViSession instrumentHandle,
											  ViInt16 graphSlct, ViBoolean plotType)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if (avopt65_invalidViInt16Range (graphSlct, 0, 8))
        return VI_ERROR_PARAMETER2;
    if (avopt65_invalidViBooleanRange (plotType))
        return VI_ERROR_PARAMETER3;
    
    Fmt (wrtBuf, "%s<C2CD%s; C2CD%s", slctGraphArr[graphSlct], plotTypeArr[plotType]);    
    
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;    
}

/*===========================================================================*/
/* Function: Graph Marker                                                    */
/* Purpose:  This function is used to switch graph marker On/Off.			 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_actGraphMkr (ViSession instrSession, ViBoolean marker)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if (avopt65_invalidViBooleanRange (marker))
        return VI_ERROR_PARAMETER2;

	if (marker)
	{
	 Fmt (wrtBuf, "%s<MK");   	
	}
	else
	{
	 Fmt (wrtBuf, "%s<MKOFF");
	}

    if ((avopt65_status = viWrite (instrSession, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;
}

/*===========================================================================*/
/* Function: DC Cal                                			                 */
/* Purpose:  This control executes DC component calibration of the 			 */
/*			 instrument.													 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt65_act_DCCal (ViSession instrumentHandle)
{
    ViUInt32 retCnt = 0;
    ViStatus avopt65_status = VI_SUCCESS;
    
    if ((avopt65_status = viWrite (instrumentHandle, "CLDC", 4, &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;    
}

/*===========================================================================*/
/* Function: Start Measurement                                               */
/* Purpose:  This function starts desired measurement mode of TRANSIENT		 */
/*           section.														 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_actMeasStart (ViSession instrumentHandle,
                                        ViInt16 function)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if (avopt65_invalidViInt16Range (function, 0, 14))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", funcMeasureArr[function]);

    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt65_status;

    return avopt65_status;
}

/*===========================================================================*/
/* Function: Check meas mode		                                         */
/* Purpose:  This function returns the mode of instrument.(CW/Transient)     */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_check_measMode (ViSession instrumentHandle,
                                         ViInt16 *returnmode)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;
    

    if ((avopt65_status = viWrite (instrumentHandle, "SETFUNC?", 8, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;

   	if (Scan (rdBuf, "%s>%d[b2]", returnmode) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt65_status;	       
    
}

/*===========================================================================*/
/* Function: Time Domain Power Result                                        */
/* Purpose:  This function returns the result of power measurement in        */
/*           the time domain.												 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_data_TDP (ViSession instrumentHandle,
                                   ViReal64 *powerLeveldBm, ViInt16 *judgement)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if ((avopt65_status = viWrite (instrumentHandle, "TDPMEAS?", 8, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;

   	if (Scan (rdBuf, "%s>%f,%d[b2]", powerLeveldBm,judgement) < 2)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt65_status;	    
}

/*===========================================================================*/
/* Function: On/Off Ratio                                                    */
/* Purpose:  This function returns the result of On/Off ratio measurement of */
/*           the burst signal in the time domain.							 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_dataOnOffRatio (ViSession instrumentHandle,
                                          ViReal64 *ONLevel, ViReal64 *OFFLevel,
                                          ViReal64 *ONOFFRaito, ViInt16 *judgement)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if ((avopt65_status = viWrite (instrumentHandle, "OORMEAS?", 8, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;

   	if (Scan (rdBuf, "%s>%f,%f,%f,%d[b2]", ONLevel, OFFLevel, ONOFFRaito, judgement) < 4)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt65_status;    
}

/*===========================================================================*/
/* Function: TD Spurious Data                                                */
/* Purpose:  This function returns the result of Time Domain Spurious		 */
/*           measurement function in the frequency domain.					 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_data_spuriTD (ViSession instrumentHandle,
                                       ViInt16 *tableSize,
                                       ViReal64 frequencyHz[], ViReal64 level[],
                                       ViInt16 judgement[])
{
    ViUInt32 i, retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if ((avopt65_status = viWrite (instrumentHandle, "TDSMEAS?", 8, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;

   	if (Scan (rdBuf, "%s>%d[b2]", tableSize) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	   	
	for (i=0;i<*tableSize;i++) {
	    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   		    return avopt65_status;
	   	if (Scan (rdBuf, "%s>%f,%f,%d[b2]", &frequencyHz[i], &level[i], &judgement[i]) < 3)
   		   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	}
    return avopt65_status;
}

/*===========================================================================*/
/* Function: F Domain Power Result                                        */
/* Purpose:  This function returns the result of power measurement in        */
/*           the time domain.												 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_data_FDP (ViSession instrumentHandle,
                                   ViReal64 *powerLeveldBm, ViInt16 *judgement)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if ((avopt65_status = viWrite (instrumentHandle, "FDPMEAS?", 8, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;

   	if (Scan (rdBuf, "%s>%f,%d[b2]", powerLeveldBm,judgement) < 2)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt65_status;	        
}

/*===========================================================================*/
/* Function: OBW Data                                                        */
/* Purpose:  This function returns the result of occupied bandwidth			 */
/*           measurement function in the frequency domain.					 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_dataOBW (ViSession instrSession, ViPReal64 OBW,
               			 		  ViPReal64 lowFreq, ViPReal64 hiFreq,
               			 		  ViInt16 *judgement)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if ((avopt65_status = viWrite (instrSession, "OBWMEAS?", 8, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrSession, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;

   	if (Scan (rdBuf, "%s>%f,%f,%f,%d[b2]", OBW, lowFreq, hiFreq, judgement) < 4)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt65_status;
}

/*===========================================================================*/
/* Function: Due to Transient Data                                           */
/* Purpose:  This function returns the result of Due to Transient			 */
/*			 measurement function in the frequency domain.					 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_data_duetoTRN (ViSession instrumentHandle,
                                        ViInt16 *numberofACP,
                                        ViReal64 ACPDatadBc[],
                                        ViInt16 judgement[])
{
    ViUInt32 i,retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if ((avopt65_status = viWrite (instrumentHandle, "DTSMEAS?", 8, &retCnt)) < 0)
        return avopt65_status;

    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;

   	if (Scan (rdBuf, "%s>%d[b2]", numberofACP) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	for (i = 0; i < *numberofACP; i++)
	{
	    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   		    return avopt65_status;
	   	if (Scan (rdBuf, "%s>%f,%d[b2]", &ACPDatadBc[i], &judgement[i]) < 2)
   		   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;		
	}
	
    return avopt65_status;    
}

/*===========================================================================*/
/* Function: Due to Modulation Data                                          */
/* Purpose:  This function returns the result of Due to Modulation			 */
/*			 measurement function in the frequency domain.					 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_data_duetoMOD (ViSession instrumentHandle,
                                        ViInt16 *numberofACP, ViReal64 ACPdBc[],
                                        ViInt16 ACPJudge[])
{
    ViUInt32 i,retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if ((avopt65_status = viWrite (instrumentHandle, "DTMMEAS?", 8, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;

   	if (Scan (rdBuf, "%s>%d[b2]", numberofACP) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	for (i = 0; i < *numberofACP; i++)
	{
	    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   		    return avopt65_status;
	   	if (Scan (rdBuf, "%s>%f,%d[b2]", &ACPdBc[i], &ACPJudge[i]) < 2)
   		   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;		
	}
    return avopt65_status;    	    
}

/*===========================================================================*/
/* Function: In Band Spurious                                                */
/* Purpose:  This function returns the result of spurious measurement. 		 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_dataSpurMeas (ViSession instrumentHandle,
                                       ViInt16 *numberofPoint,
                                       ViReal64 frequencyHz[],
                                       ViReal64 level[], ViInt16 judge[])
{
    ViUInt32 i, retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if ((avopt65_status = viWrite (instrumentHandle, "SPRMEAS?", 8, &retCnt)) < 0)
        return avopt65_status;
        
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;

   	if (Scan (rdBuf, "%s>%d[b2]", numberofPoint) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	   	
	for (i=0;i<*numberofPoint;i++) {
	    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   		    return avopt65_status;
	   	
	   	if (Scan (rdBuf, "%s>%f,%f,%d[b2]", &frequencyHz[i], &level[i], &judge[i]) < 3)
   		   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	}
    return avopt65_status;
}

/*===========================================================================*/
/* Function: Out Band Spurious                                               */
/* Purpose:  This function returns the result of spurious measurement. 		 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_data_spuriOB (ViSession instrumentHandle,
                                       ViInt16 *numberofPoint,
                                       ViReal64 frequencyHz[],
                                       ViReal64 level[], ViInt16 judgement[])
{
    ViUInt32 i, retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if ((avopt65_status = viWrite (instrumentHandle, "FDSMEAS?", 8, &retCnt)) < 0)
        return avopt65_status;
        
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;

   	if (Scan (rdBuf, "%s>%d[b2]", numberofPoint) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	   	
	for (i=0;i<*numberofPoint;i++) {
	    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   		    return avopt65_status;
	   	
	   	if (Scan (rdBuf, "%s>%f,%f,%d[b2]", &frequencyHz[i], &level[i], &judgement[i]) < 3)
   		   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	}
    return avopt65_status;		
}

/*===========================================================================*/
/* Function: CDP Coefficient Measurement Data                                */
/* Purpose:  This function returns the result of CDP coefficient  			 */
/*			 measurement.											 		 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt65_dataCDPCoef (ViSession instrumentHandle,
                                       ViReal64 *tmAlgnErr, ViReal64 *crrFreqErrHz,
                                       ViReal64 *crrFreqErrPPM, ViReal64 *mltplRho, 
                                       ViInt16 *offsetPN, ViReal64 *magErr,
                                       ViReal64 *phsErr, ViReal64 *errVctrMag,
                                       ViReal64 *orgnOfstIQ, ViReal64 *errSigRatio,
                                       ViReal64 *txPower, ViReal64 *avgPower,
                                       ViReal64 *peakCDE)
{
    ViUInt32 i, retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if ((avopt65_status = viWrite (instrumentHandle, "C2PCTAU?", 8, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;
   	if (Scan (rdBuf, "%s>%f", tmAlgnErr) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    if ((avopt65_status = viWrite (instrumentHandle, "C2PCCFER?", 9, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;
   	if (Scan (rdBuf, "%s>%f,%f", crrFreqErrHz, crrFreqErrPPM) < 2)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    if ((avopt65_status = viWrite (instrumentHandle, "C2PCRHO?", 8, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;
   	if (Scan (rdBuf, "%s>%f", mltplRho) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    if ((avopt65_status = viWrite (instrumentHandle, "C2PCPNOFS?", 10, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;
   	if (Scan (rdBuf, "%s>%d[b2]", offsetPN) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    if ((avopt65_status = viWrite (instrumentHandle, "C2PCMAG?", 8, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;
   	if (Scan (rdBuf, "%s>%f", magErr) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    if ((avopt65_status = viWrite (instrumentHandle, "C2PCPHE?", 8, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;
   	if (Scan (rdBuf, "%s>%f", phsErr) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    if ((avopt65_status = viWrite (instrumentHandle, "C2PCEVM?", 8, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;
   	if (Scan (rdBuf, "%s>%f", errVctrMag) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    if ((avopt65_status = viWrite (instrumentHandle, "C2PCIQOFS?", 10, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;
   	if (Scan (rdBuf, "%s>%f", orgnOfstIQ) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    if ((avopt65_status = viWrite (instrumentHandle, "C2PCERPOW?", 10, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;
   	if (Scan (rdBuf, "%s>%f", errSigRatio) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    if ((avopt65_status = viWrite (instrumentHandle, "C2PCTXPOW?", 10, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;
   	if (Scan (rdBuf, "%s>%f", txPower) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    if ((avopt65_status = viWrite (instrumentHandle, "C2PCAVGPOW?", 11, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;
   	if (Scan (rdBuf, "%s>%f", avgPower) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    if ((avopt65_status = viWrite (instrumentHandle, "C2PCPKCDE?", 10, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;
   	if (Scan (rdBuf, "%s>%f", peakCDE) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    return avopt65_status;
}

/*===========================================================================*/
/* Function: CDP Coefficient Marker Data                                     */
/* Purpose:  This function returns graphic marker's value of CDP coefficient */
/*			 measurement.											 		 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt65_marker_dataCDPCoef (ViSession instrumentHandle,
                                              ViInt16 grphType, ViInt16 mkrPos,
                                              ViReal64 *mkrData)
{
  	ViUInt32 i, retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE], rdBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;    

	if (avopt65_invalidViInt16Range (grphType, 0, 4))
        return VI_ERROR_PARAMETER2;
	if (avopt65_invalidViInt16Range (mkrPos, 0, 127))
        return VI_ERROR_PARAMETER3;
        
    Fmt (wrtBuf, "%s<C2PCMK %d", mkrPos);    
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt65_status;
    
    Fmt (wrtBuf, "%s<C2PCMK%s?", sclYCDPCFArr[grphType]);    
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;
   	if (Scan (rdBuf, "%s>%f", mkrData) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    return avopt65_status;
}

/*===========================================================================*/
/* Function: CDP Measurement Data                                            */
/* Purpose:  This function returns the result of CDP measurement.  			 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt65_dataCDP (ViSession instrumentHandle,
                                   ViReal64 *estRho, ViReal64 *tmAlgnErr,
                                   ViReal64 *crrFreqErr, ViReal64 *magErr,
                                   ViReal64 *phsErr, ViReal64 *errVctrMag,
                                   ViReal64 *orgOfstIQ, ViReal64 *dlyPN)
{
    ViUInt32 i, retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if ((avopt65_status = viWrite (instrumentHandle, "C2CDTOTAL?", 10, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;
   	if (Scan (rdBuf, "%s>%f,%f,%f,%f,%f,%f,%f,%f", 
   			estRho, tmAlgnErr, crrFreqErr, magErr, phsErr, errVctrMag, orgOfstIQ, dlyPN) < 8)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
	return avopt65_status;
}

/*===========================================================================*/
/* Function: CDP Marker Data			                                     */
/* Purpose:  This function returns graphic marker's value of CDP 			 */
/*           measurement.													 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt65_marker_dataCDP (ViSession instrumentHandle,
                                          ViInt16 viewChnl,
                                          ViReal64 *cdPwr, ViReal64 *rho,
                                          ViReal64 *pwrdBm, ViReal64 *pwrW,
                                          ViReal64 *valEVM)
{
  	ViUInt32 i, retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE], rdBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;    

	if (avopt65_invalidViInt16Range (viewChnl, 0, 6))
        return VI_ERROR_PARAMETER2;
   
    Fmt (wrtBuf, "%s<C2VWCH %s", viewChnlArr[viewChnl]);
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt65_status;

    if ((avopt65_status = viWrite (instrumentHandle, "C2CDMKPOW?", 10, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;
   	if (Scan (rdBuf, "%s>%f", cdPwr) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    if ((avopt65_status = viWrite (instrumentHandle, "C2CDMKRHO?", 10, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;
   	if (Scan (rdBuf, "%s>%f", rho) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    if ((avopt65_status = viWrite (instrumentHandle, "C2CDMKPWR?", 10, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;
   	if (Scan (rdBuf, "%s>%f,%f", pwrdBm, pwrW) < 2)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    if ((avopt65_status = viWrite (instrumentHandle, "C2CDMKEVM?", 10, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;
   	if (Scan (rdBuf, "%s>%f", valEVM) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
	return avopt65_status;
}

/*===========================================================================*/
/* Function: CDP EVM Marker Data		                                     */
/* Purpose:  This function returns marker's value of EVM graph.				 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt65_EVM_dataCDP (ViSession instrumentHandle,
                                       ViInt16 mkrPos, ViReal64 *valEVM)
{
  	ViUInt32 i, retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE], rdBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;    

	if (avopt65_invalidViInt16Range (mkrPos, 0, 200))
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuf, "%s<C2EVMMKSYM %d", mkrPos);    
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt65_status;
    
    if ((avopt65_status = viWrite (instrumentHandle, "C2EVMMKEVM?", 11, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;
   	if (Scan (rdBuf, "%s>%f", valEVM) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    return avopt65_status;
}

/*===========================================================================*/
/* Function: CDP IQ Graph Data		                                     	 */
/* Purpose:  This function returns marker's value of IQ graph.				 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt65_graphIQ_dataCDP (ViSession instrumentHandle,
                                           ViInt32 chipN,
                                           ViReal64 *phaseI, ViReal64 *phaseQ)
{
  	ViUInt32 i, retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE], rdBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;    

    Fmt (wrtBuf, "%s<C2MKCHIP %d", chipN);    
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt65_status;
    
    if ((avopt65_status = viWrite (instrumentHandle, "C2MKI?", 6, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;
   	if (Scan (rdBuf, "%s>%f", phaseI) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    if ((avopt65_status = viWrite (instrumentHandle, "C2MKQ?", 6, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;
   	if (Scan (rdBuf, "%s>%f", phaseQ) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
	return avopt65_status;
}

/*===========================================================================*/
/* Function: CDP Error Graph Data	                                     	 */
/* Purpose:  This function returns the marker's	value of E.V.M. vs chip 	 */
/*			 graph or mag error vs chip graph.								 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt65_graphError_dataCDP (ViSession instrumentHandle,
                                              ViInt16 mkrPos,
                                              ViInt16 *chipN, ViReal64 *errDt)
{
  	ViUInt32 i, retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE], rdBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;    

    Fmt (wrtBuf, "%s<C2GMK %d", mkrPos);    
    if ((avopt65_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt65_status;
    
    if ((avopt65_status = viWrite (instrumentHandle, "C2MKCHIP?", 9, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;
   	if (Scan (rdBuf, "%s>%d[b2]", chipN) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
    if ((avopt65_status = viWrite (instrumentHandle, "C2MKERR?", 8, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;
   	if (Scan (rdBuf, "%s>%f", errDt) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    
	return avopt65_status;
}

/*===========================================================================*/
/* Function: TX Power                                                        */
/* Purpose:  This function returns the result of TX power measurement. 		 */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_dataTXPow (ViSession instrumentHandle,
                                     ViReal64 *powerLeveldBm,
                                     ViReal64 *powerLevelW,
                                     ViReal64 *peakFactordB)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if ((avopt65_status = viWrite (instrumentHandle, "TXPWR?", 6, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;

   	if (Scan (rdBuf, "%s>%f,%f,%f", powerLeveldBm, powerLevelW,peakFactordB) < 3)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt65_status;
}

/*===========================================================================*/
/* Function: CCDF			                                                 */
/* Purpose:  This function returns the result of CCDF measurement.			 */
/*===========================================================================*/
ViStatus _VI_FUNC avopt65_dataCCDF (ViSession instrumentHandle,
                                    ViReal64 *pkFact, ViReal64 *avgPwr,
                                    ViReal64 *pwr10, ViReal64 *pwr1,
                                    ViReal64 *pwr01, ViReal64 *pwr001,
                                    ViReal64 *pwr0001, ViReal64 *pwr00001)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

    if ((avopt65_status = viWrite (instrumentHandle, "C2CCDF?", 7, &retCnt)) < 0)
        return avopt65_status;
    if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;

   	if (Scan (rdBuf, "%s>%f,%f,%f,%f,%f,%f,%f,%f", pkFact, avgPwr, pwr10, pwr1,
   			  pwr01, pwr001, pwr0001, pwr00001) < 8)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt65_status;
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus CVIFUNC avopt65_close (ViSession instrSession)
{
    avopt65_instrRange instrPtr;
    ViSession rmSession;
    ViStatus avopt65_status = VI_SUCCESS;

    if ((avopt65_status = viGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession)) < 0)
        return avopt65_status;
    if ((avopt65_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return avopt65_status;
            
    if (instrPtr != NULL) 
        free (instrPtr);
    
    avopt65_status = viClose (instrSession);
    viClose (rmSession);

    return avopt65_status;
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean avopt65_invalidViBooleanRange (ViBoolean val)
{
    return ((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Signed Integer Value Out Of Range - ViInt16               */
/* Purpose:  This function checks a short signed integer value to see if it  */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avopt65_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avopt65_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Unsigned Char Value Out Of Range - ViUInt8                      */
/* Purpose:  This function checks an unsigned char value to see if it        */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avopt65_invalidViUInt8Range (ViUInt8 val, ViUInt8 min, ViUInt8 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Unsigned Integer Value Out Of Range - ViUInt16            */
/* Purpose:  This function checks a short unsigned integer value to see if it*/  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avopt65_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32             */
/* Purpose:  This function checks a long unsigned integer value to see if it */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avopt65_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Float) Value Out Of Range - ViReal32                      */
/* Purpose:  This function checks a real (float) value to see if it lies     */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean avopt65_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean avopt65_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*================================================================================*/
/* Function: Wait On VISA Event                                                   */
/* Purpose:  This function waits for the occurrence of an enabled VISA Event. The */
/*           event type must be enabled before entering this function. Any        */
/*           programmatic commands, actions, or conditions, necessary to generate */
/*           the specified event type, must be satisified before entering this    */
/*           function. The function will wait for the specified timeout and then  */
/*           return. If the specified event is received then the function will    */
/*           return VI_SUCCESS (0), otherwise the status code for the generated   */
/*           error will be returned. In either case the function will disable the */
/*           event type and deallocate the event handle passed from viWaitOnEvent */
/*           before returning. WARNING: If VI_TMO_INFINITE is passed in as the    */
/*           timeout this function WILL NOT return until the specified event is   */
/*           received, if the event is not received the function will not return  */
/*           and it will be necessary to terminate CVI in order to regain control.*/
/*================================================================================*/
ViStatus avopt65_waitOnVisaEvent (ViSession instrSession, ViEventType eventTypeIn, 
            ViUInt32 timeoutIn, ViPUInt16 STB)
{            
    ViStatus avopt65_status = VI_SUCCESS, tempStatus = VI_SUCCESS;
    ViEventType eventTypeOut = 0;
    ViEvent eventHandle = 0;
    
    /* For debug purposes we want to be able to see the status returned by        */
    /* viDisableEvent and viClose if one of the internal functions fails but do   */
    /* not want to return that value as that is not where the initial error       */
    /* occurs in the function, so we assign it to tempStatus.                     */
    
    if ((avopt65_status = viWaitOnEvent (instrSession, eventTypeIn, timeoutIn, 
                                        &eventTypeOut, &eventHandle)) < 0) {
        tempStatus = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE);
        return avopt65_status;
    } 
    
    if (eventTypeIn == VI_EVENT_SERVICE_REQ)
        if ((avopt65_status = viReadSTB (instrSession, STB)) < 0) {
            tempStatus = viClose (eventHandle);
            tempStatus = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE);
            return avopt65_status;
        }   
        
    if ((avopt65_status = viClose (eventHandle)) < 0) {             
        tempStatus = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE);
        return avopt65_status;                 
    }
    
    if ((avopt65_status = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE)) < 0)
        return avopt65_status;                                                   

    return avopt65_status;                                    
}    

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the avopt65_init function.  When  */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from avopt65_init to      */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus avopt65_initCleanUp (ViSession openRMSession,
                ViPSession openInstrSession, ViStatus currentStatus)
{
    avopt65_instrRange instrPtr;
    
    if (viGetAttribute (*openInstrSession, VI_ATTR_USER_DATA, &instrPtr) >= 0)
        if (instrPtr != NULL) 
            free (instrPtr);

    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;
    
    return currentStatus;
}

/*****************************************************************************/
/*----------- INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINES HERE -------------*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the avopt65_reset operation and by the    */
/*           avopt65_init function if the reset option has not been          */
/*           selected.  This function is useful for configuring any          */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/                                    
/*===========================================================================*/
ViStatus avopt65_defaultInstrSetup (ViSession instrSession)
{
    ViStatus avopt65_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    avopt65_instrRange instrPtr;
    
    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
    if (avopt65_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr))
        return avopt65_status;
    
    if (instrPtr == NULL) 
        instrPtr = malloc (sizeof (struct avopt65_statusDataRanges));

    instrPtr -> traceRange = 1001;
    instrPtr -> harmonics = 3;
    Fmt (instrPtr -> instrDriverRevision, "%s<%s", avopt65_REVISION);
    
    if ((avopt65_status = viSetAttribute (instrSession, VI_ATTR_USER_DATA, 
                            (ViUInt32)instrPtr)) < 0)
        return avopt65_status;                                       

    return avopt65_status;
}

/*===========================================================================*/
/* Function: Read Graph Data				                                 */
/* Purpose:  This function returns the graph data			 				 */
/*===========================================================================*/

ViStatus avopt65_readDoubleArray (ViSession instrumentHandle, ViChar *command,
								  ViInt16 *dataSize, ViReal64 data[])
{
	ViInt16  n;
	ViUInt32 retCnt = 0;
    ViChar   rdBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

	if ((avopt65_status = viWrite (instrumentHandle, command, StringLength (command), &retCnt)) < 0)
       	return avopt65_status;
	if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;
   	if (Scan (rdBuf, "%s>%d[b2]", dataSize) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   			
   	n = *dataSize;
   	while (--n>=0)
   	{
	   	if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    	return avopt65_status;
   		if (Scan (rdBuf, "%s>%f", data++) < 1)
   			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	}
	
	return avopt65_status;
}

ViStatus avopt65_readShortArray (ViSession instrumentHandle, ViChar *command,
								 ViInt16 *dataSize, ViInt16 data[])
{
	ViInt16  n;
	ViUInt32 retCnt = 0;
    ViChar   rdBuf[BUFFER_SIZE];
    ViStatus avopt65_status = VI_SUCCESS;

	if ((avopt65_status = viWrite (instrumentHandle, command, StringLength (command), &retCnt)) < 0)
       	return avopt65_status;
	if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt65_status;
   	if (Scan (rdBuf, "%s>%d[b2]", dataSize) < 1)
   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   			
   	n = *dataSize;
   	while (--n>=0)
   	{
	   	if ((avopt65_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    	return avopt65_status;
   		if (Scan (rdBuf, "%s>%d[b2]", data++) < 1)
   			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	}
	
	return avopt65_status;
}

ViStatus avopt65_dataGraph (ViSession instrumentHandle, ViInt16 grphSlct,
                            ViInt16 *iDataSize, ViReal64 iData[],
                            ViInt16 *qDataSize, ViReal64 qData[],
                            ViInt16 *xDataSize, ViInt16  xData[],
                            ViInt16 *yDataSize, ViReal64 yData[])
{
    ViStatus avopt65_status = VI_SUCCESS;
    
    if (grphSlct<=6)
    {
    
    	if ((avopt65_status = avopt65_readDoubleArray (instrumentHandle, "GPHI?", iDataSize, iData)) < 0)
    		return avopt65_status;
    		
    	if ((avopt65_status = avopt65_readDoubleArray (instrumentHandle, "GPHQ?", qDataSize, qData)) < 0)
    		return avopt65_status;
    }
    
    else
    {
    	*iDataSize = 0;
    	*qDataSize = 0;
    }
    
    if (grphSlct>=4)
    {
    	if ((avopt65_status = avopt65_readShortArray (instrumentHandle, "GPHX?", xDataSize, xData)) < 0)
    		return avopt65_status;
    }
    
    else
    	*xDataSize = 0;
	
    if (grphSlct>=7)
    {
    	if ((avopt65_status = avopt65_readDoubleArray (instrumentHandle, "GPHY?", yDataSize, yData)) < 0)
    		return avopt65_status;
    }
    
    else
    	*yDataSize = 0;
	
	return avopt65_status;
}

/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/
