#include <cvidef.h>

/*= R3267 Series - Modulation Spectrum Analyzes Include File ================*/

/*===========================================================================*/
/*  Please do not use global variables or arrays in the include file of      */
/*  instrument drivers that will be submitted for inclusion into the         */
/*  LabWindows Instrument Driver Library.                                    */
/*===========================================================================*/
     
#ifndef __avopt65_HEADER
#define __avopt65_HEADER

#include <vpptype.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/*****************************************************************************/
/*= Instrument Driver Specific Error/Warning Codes ==========================*/
/*****************************************************************************/
#define VI_ERROR_INSTR_FILE_OPEN                (_VI_ERROR+0x3FFC0800L)
#define VI_ERROR_INSTR_FILE_WRITE               (_VI_ERROR+0x3FFC0801L)
#define VI_ERROR_INSTR_INTERPRETING_RESPONSE    (_VI_ERROR+0x3FFC0803L)
#define VI_ERROR_INSTR_PARAMETER9               (_VI_ERROR+0x3FFC0809L)
#define VI_ERROR_INSTR_PARAMETER10              (_VI_ERROR+0x3FFC080AL)
#define VI_ERROR_INSTR_PARAMETER11              (_VI_ERROR+0x3FFC080BL)
#define VI_ERROR_INSTR_PARAMETER12              (_VI_ERROR+0x3FFC080CL)
#define VI_ERROR_INSTR_PARAMETER13              (_VI_ERROR+0x3FFC080DL)
#define VI_ERROR_INSTR_PARAMETER14              (_VI_ERROR+0x3FFC080EL)
#define VI_ERROR_INSTR_PARAMETER15              (_VI_ERROR+0x3FFC080FL)
#define VI_ERROR_INSTR_PARAMETER16              (_VI_ERROR+0x3FFC0810L)
#define VI_ERROR_INSTR_PARAMETER17              (_VI_ERROR+0x3FFC0811L)  
/*****************************************************************************/
/*= Define Instrument Specific Error/Warning Codes Here =====================*/
/*****************************************************************************/
#define VI_WARNING_INSTR_OFFSET                           (0x3FFC0900L)
#define VI_ERROR_INSTR_OFFSET                   (_VI_ERROR+0x3FFC0900L)

#define avopt65_ERROR_INVALID_CONFIGURATION      (VI_ERROR_INSTR_OFFSET + 0xF0L)

/*****************************************************************************/
/*= GLOBAL USER-CALLABLE FUNCTION DECLARATIONS (Exportable Functions) =======*/
/*****************************************************************************/
ViStatus _VI_FUNC avopt65_confStdSet (ViSession instrumentHandle,
                                      ViInt16 bandClass, ViBoolean link,
                                      ViReal64 offsetLeveldB,
                                      ViBoolean frequencyInput,
                                      ViBoolean IQInverse,
                                      ViBoolean contAutoLevelSet);

ViStatus CVIFUNC avopt65_ChEdt (ViSession instrumentHandle, ViBoolean settoSTD,
                                ViBoolean editLink, ViInt16 editChnl,
                                ViInt16 startCHNum, ViInt16 stopCHNum, 
                                ViReal64 baseFreqHz, ViReal64 channelSpaceHz, 
                                ViInt16 CHOffset);

ViStatus CVIFUNC avopt65_ch_Enable (ViSession instrumentHandle,
                                    ViBoolean channel1, ViBoolean channel2,
                                    ViBoolean channel3, ViBoolean channel4,
                                    ViBoolean channel5, ViBoolean channel6,
                                    ViBoolean channel7, ViBoolean channel8,
                                    ViBoolean channel9);

ViStatus CVIFUNC avopt65_confTrgConTDP (ViSession instrumentHandle,
                                        ViInt16 triggerSource,
                                        ViBoolean trgSlope, ViInt16 triggerLevel,
                                        ViInt16 triggerPosition,
                                        ViReal64 delayTimes);

ViStatus CVIFUNC avopt65_windowSetTDP (ViSession instrumentHandle,
                                       ViBoolean settoSTD, ViBoolean window,
                                       ViReal64 winPosS, ViReal64 winWidS);

ViStatus CVIFUNC avopt65_template_Set_TDP (ViSession instrumentHandle,
                                           ViBoolean template, ViReal64 shiftXS,
                                           ViReal64 shiftYdB);

ViStatus CVIFUNC avopt65_template_editTDP (ViSession instrumentHandle,
                                           ViInt16 operation,
                                           ViBoolean templateSelect,
                                           ViReal64 templateTimes,
                                           ViReal64 templateLevel);

ViStatus CVIFUNC avopt65_parameter_setTDP (ViSession instrumentHandle,
                                           ViBoolean settoSTD, ViInt16 detector,
                                           ViInt16 displayUnit,
                                           ViBoolean TMPLCoupleofPWR,
                                           ViReal64 templateLimit,
                                           ViBoolean judgement,
                                           ViReal64 upperLimit,
                                           ViReal64 lowerLimit);

ViStatus CVIFUNC avopt65_trigger_setOOR (ViSession instrumentHandle,
                                         ViInt16 triggerSource,
                                         ViBoolean trgSlope,
                                         ViInt16 triggerLevel,
                                         ViInt16 triggerPosition,
                                         ViReal64 delayTimes);

ViStatus CVIFUNC avopt65_window_setOOR (ViSession instrumentHandle,
                                        ViBoolean settoSTD, ViBoolean window,
                                        ViReal64 onPositionS, ViReal64 onWidthS,
                                        ViReal64 offPositionS,
                                        ViReal64 offWidthS);

ViStatus CVIFUNC avopt65_param_setOOR (ViSession instrumentHandle,
                                       ViBoolean settoSTD, ViInt16 detector,
                                       ViInt16 displayUnit, ViBoolean judgement,
                                       ViReal64 upperLimit);

ViStatus CVIFUNC avopt65_trigger_setTDS (ViSession instrumentHandle,
                                         ViInt16 triggerSource,
                                         ViBoolean trgSlope,
                                         ViInt16 triggerLevel,
                                         ViInt16 triggerPosition,
                                         ViReal64 delayTimes);

ViStatus CVIFUNC avopt65_table_editTDS (ViSession instrumentHandle,
                                        ViInt16 operation, ViInt16 tableNumber,
                                        ViBoolean freqType, ViReal64 carrierFreq,
                                        ViReal64 frequency, ViReal64 level);

ViStatus CVIFUNC avopt65_param_setTDS (ViSession instrumentHandle,
                                       ViBoolean settoSTD, ViInt16 detector,
                                       ViBoolean result, ViReal64 peakMKRYDelta, 
                                       ViReal64 multiplier, ViInt16 displayUnit,
                                       ViBoolean judgement, ViBoolean preselecter);

ViStatus _VI_FUNC avopt65_confGateTrg (ViSession instrumentHandle,
                                       ViBoolean gateSource, ViBoolean set_toSTD,
                                       ViBoolean measurementMode,
                                       ViReal64 gatePositions,
                                       ViReal64 gateWidths, ViInt16 trgSource,
                                       ViInt16 triggerPosition,
                                       ViReal64 delayTimes, ViBoolean trgSlope,
                                       ViInt16 triggerLevel,
                                       ViBoolean gatedSweep, ViInt16 detector);

ViStatus CVIFUNC avopt65_window_setFDP (ViSession instrumentHandle,
                                        ViBoolean settoSTD, ViBoolean window,
                                        ViReal64 positionHz, ViReal64 widthHz);

ViStatus CVIFUNC avopt65_parameter_setFDP (ViSession instrumentHandle,
                                           ViBoolean settoSTD, ViInt16 detector,
                                           ViInt16 displayUnit,
                                           ViBoolean gatedSweep,
                                           ViBoolean judgement,
                                           ViReal64 upperLimit,
                                           ViReal64 lowerLimit);

ViStatus CVIFUNC avopt65_parameter_setOBW (ViSession instrumentHandle,
                                           ViBoolean settoSTD, ViReal64 OBW,
                                           ViInt16 detector, ViBoolean judgement,
                                           ViReal64 upperLimit,
                                           ViReal64 lowerLimit);

ViStatus CVIFUNC avopt65_parameter_setDT (ViSession instrumentHandle,
                                          ViBoolean settoSTD, ViInt16 detector,
                                          ViInt16 displayUnit,
                                          ViBoolean TMPLCoupleofPWR,
                                          ViReal64 templateLimit,
                                          ViBoolean judgement,
                                          ViBoolean freqSetting, ViInt16 result,
                                          ViBoolean refPower,
                                          ViReal64 symbolRate, ViReal64 rolloff);

ViStatus CVIFUNC avopt65_template_setDT (ViSession instrumentHandle,
                                         ViBoolean template, ViReal64 shiftXHz,
                                         ViReal64 shiftYdB, ViReal64 deltaXHz);

ViStatus CVIFUNC avopt65_template_editDT (ViSession instrumentHandle,
                                          ViInt16 operation,
                                          ViReal64 templateFreqHz,
                                          ViReal64 templateLeveldB);

ViStatus CVIFUNC avopt65_marker_editDT (ViSession instrumentHandle,
                                        ViInt16 operation,
                                        ViInt16 refMarkerType,
                                        ViReal64 refBandWidthHz,
                                        ViBoolean offsetMkrType,
                                        ViReal64 offsetFreqHz,
                                        ViReal64 bandWidthHz,
                                        ViReal64 limitPowerdB);

ViStatus CVIFUNC avopt65_parameter_setDM (ViSession instrumentHandle,
                                          ViBoolean settoSTD,
                                          ViBoolean freqSetting,
                                          ViInt16 detector, ViInt16 result,
                                          ViBoolean refPower,
                                          ViInt16 displayUnit,
                                          ViBoolean TMPLCoupleofPWR,
                                          ViBoolean judgement,
                                          ViReal64 templateLimit,
                                          ViReal64 symbolRate, ViReal64 rolloff);

ViStatus CVIFUNC avopt65_template_setDM (ViSession instrumentHandle,
                                         ViBoolean template, ViReal64 shiftXHz,
                                         ViReal64 shiftYdB, ViReal64 deltaXHz);

ViStatus CVIFUNC avopt65_template_editDM (ViSession instrumentHandle,
                                          ViInt16 operation,
                                          ViReal64 templateFreqHz,
                                          ViReal64 templateLeveldB);

ViStatus CVIFUNC avopt65_marker_editDM (ViSession instrumentHandle,
                                        ViInt16 operation,
                                        ViInt16 refMarkerType,
                                        ViReal64 refBandWidthHz,
                                        ViBoolean offsetMkrType,
                                        ViReal64 offsetFreqHz,
                                        ViReal64 bandWidthHz,
                                        ViReal64 limitPowerdB);

ViStatus CVIFUNC avopt65_parameter_setINSPR (ViSession instrumentHandle,
                                             ViBoolean settoSTD,
                                             ViBoolean freqSetting,
                                             ViInt16 detector,
                                             ViBoolean refPower,
                                             ViReal64 peakMkrYDelta,
                                             ViInt16 result, ViInt16 displayUnit,
                                             ViBoolean TMPLCoupleofPWR,
                                             ViReal64 templateLimit,
                                             ViBoolean judgement);

ViStatus CVIFUNC avopt65_template_setINSPR (ViSession instrumentHandle,
                                            ViBoolean template,
                                            ViReal64 shiftXHz, ViReal64 shiftYdB,
                                            ViReal64 deltaXHz);

ViStatus CVIFUNC avopt65_template_editINSPR (ViSession instrumentHandle,
                                             ViInt16 operation,
                                             ViReal64 templateFreqHz,
                                             ViReal64 templateLeveldB);

ViStatus CVIFUNC avopt65_marker_editINSPR (ViSession instrumentHandle,
                                           ViInt16 operation,
                                           ViBoolean refMarkerType,
                                           ViReal64 refBandWidthHz,
                                           ViReal64 startFreqHz,
                                           ViReal64 stopFreqHz,
                                           ViReal64 limitPowerdB);

ViStatus CVIFUNC avopt65_parameter_setOUTSPR (ViSession instrumentHandle,
                                              ViBoolean settoSTD,
                                              ViInt16 detector,
                                              ViReal64 peakMkrYDelta,
                                              ViInt16 displayUnit,
                                              ViBoolean judgement,
                                              ViBoolean preselector);

ViStatus CVIFUNC avopt65_table_setOUTSPR (ViSession instrumentHandle,
                                          ViInt16 tableNumber,
                                          ViBoolean loadTable,
                                          ViBoolean saveTable, ViInt16 tableSize,
                                          ViReal64 startFreqHz[],
                                          ViReal64 stopFreqHz[], ViReal64 RBW[],
                                          ViReal64 VBW[], ViReal64 sweepTime[],
                                          ViReal64 limitLevel[]);

ViStatus CVIFUNC avopt65_table_editOUTSPR (ViSession instrumentHandle,
                                           ViInt16 operation,
                                           ViInt16 tableNumber,
                                           ViReal64 startFreqHz,
                                           ViReal64 stopFreqHz, ViReal64 RBW,
                                           ViReal64 VBW, ViReal64 sweepTime,
                                           ViReal64 limitLevel);

ViStatus CVIFUNC avopt65_confYCodeDomPow (ViSession instrumentHandle,
                                          ViBoolean tauOffset,
                                          ViReal64 tauOffsetValus,
                                          ViBoolean equalFilter,
                                          ViBoolean measMode,
                                          ViReal64 thresholddB,
                                          ViInt16 measRangechip,
                                          ViBoolean PNSearchMode,
                                          ViInt16 PNOffset, ViBoolean trgSlope,
                                          ViReal64 extTrigDelaychip,
                                          ViInt16 pagingChannel,
                                          ViInt16 trafficChannel,
                                          ViBoolean phaseUnit,
                                          ViBoolean powerUnit,
                                          ViBoolean freqMeasRange,
                                          ViInt16 triggerSource);

ViStatus _VI_FUNC avopt65_parameter_setCDPCoef (ViSession instrumentHandle,
                                                ViInt32 measRange,
                                                ViReal64 tauOffset,
                                                ViBoolean complementaryFilter,
                                                ViReal64 rolloffFactor,
                                                ViBoolean equalizingFilter,
                                                ViBoolean PNOffsetSearchMode,
                                                ViInt16 PNOffset,
                                                ViInt16 carrierFrequencySearch,
                                                ViInt16 triggerSource,
                                                ViBoolean EXTTriggerSlope,
                                                ViInt16 threshold,
                                                ViBoolean channelDefine,
                                                ViInt16 walshCodeLength,
                                                ViInt16 QOF,
                                                ViBoolean paleyOrder);

ViStatus _VI_FUNC avopt65_scale_setCDPCoef (ViSession instrumentHandle,
                                            ViInt16 format, ViBoolean display,
                                            ViInt16 YScale, ViInt16 YScaleUpper,
                                            ViBoolean ydiv, ViInt16 tablePage);

ViStatus _VI_FUNC avopt65_chDefTbl_editCDPCoef (ViSession instrumentHandle,
                                                ViInt16 totalChannelNumber,
                                                ViInt16 editChannel,
                                                ViInt16 channelName,
                                                ViInt16 radioConfigulation,
                                                ViInt32 dataRate, ViInt16 QOF,
                                                ViInt16 walshCodeNumber);

ViStatus _VI_FUNC avopt65_measOption_setCDPCoef (ViSession instrumentHandle,
                                                 ViInt16 CDPGraphPlotType,
                                                 ViInt16 powerUnit,
                                                 ViReal64 pilotChannelPower,
                                                 ViBoolean deltaTau,
                                                 ViBoolean deltaTheta,
                                                 ViBoolean codeDomainError,
                                                 ViBoolean signalPower,
                                                 ViBoolean EVM,
                                                 ViBoolean fixedSymbolsLevel);

ViStatus _VI_FUNC avopt65_parameter_setCDP (ViSession instrumentHandle,
                                            ViBoolean measMode,
                                            ViInt16 measRange, ViInt16 threshold,
                                            ViBoolean PNDelaySearchMode,
                                            ViInt16 PNDelay,
                                            ViInt16 triggerSource,
                                            ViBoolean EXTTriggerSlope,
                                            ViReal64 EXTTriggerDelay,
                                            ViBoolean freqMeasRange);

ViStatus _VI_FUNC avopt65_view_setCDP (ViSession instrumentHandle,
                                       ViInt16 display, ViInt16 viewChannel,
                                       ViBoolean CDPYdiv);

ViStatus _VI_FUNC avopt65_channel_setCDP (ViSession instrumentHandle,
                                          ViInt16 operationMode,
                                          ViBoolean SCH1WalshFunction,
                                          ViInt16 SCH1RepetitionFactor,
                                          ViBoolean SCH2WalshFunction,
                                          ViInt16 SCH2RepetitionFactor);

ViStatus _VI_FUNC avopt65_parameter_setTxPower (ViSession instrumentHandle,
                                                ViInt16 triggerMode,
                                                ViBoolean EXTTriggerSlope,
                                                ViReal64 EXTTriggerDelay);

ViStatus _VI_FUNC avopt65_scale_setCCDF (ViSession instrumentHandle,
                                         ViInt16 XScaleMax, ViInt16 XScaleRange,
                                         ViBoolean powerUnit);

ViStatus _VI_FUNC avopt65_parameter_setCCDF (ViSession instrumentHandle,
                                             ViBoolean triggerMode,
                                             ViBoolean triggerSlope,
                                             ViReal64 triggerDelay,
                                             ViInt32 measLength);

ViStatus CVIFUNC avopt65_average_set (ViSession instrSession, ViInt16 function,
                ViInt16 AVGTimes);

ViStatus _VI_FUNC avopt65_Yscale_set (ViSession instrumentHandle,
                                      ViInt16 function, ViInt16 YScale);

ViStatus CVIFUNC avopt65_execute_auto_lvl (ViSession instrumentHandle,
                                           ViInt16 function);

ViStatus CVIFUNC avopt65_actYDispCtrl (ViSession instrSession, ViInt16 zoomingFunction,
				ViReal64 windowPositions, ViReal64 windowWidths, ViBoolean verticalZoom,
				ViInt16 YScale);

ViStatus _VI_FUNC avopt65_act_DCCal (ViSession instrumentHandle);

ViStatus CVIFUNC avopt65_actMeasStart (ViSession instrumentHandle, ViInt16 function);

ViStatus _VI_FUNC avopt65_selectGraphTypeCDPCoef (ViSession instrumentHandle,
                                                  ViInt16 graphSelection,
                                                  ViBoolean plotType);

ViStatus _VI_FUNC avopt65_selectGraphTypeCDP (ViSession instrumentHandle,
											  ViInt16 graphSelection,
                                              ViBoolean plotType);

ViStatus CVIFUNC avopt65_actGraphMkr (ViSession instrSession,
                ViBoolean marker);

ViStatus CVIFUNC avopt65_actYLimitLine (ViSession instrSession,
                ViInt16 typeSelection, ViBoolean tableType);
          
ViStatus CVIFUNC avopt65_check_measMode (ViSession instrumentHandle,
                                         ViInt16 *returnmode);

ViStatus CVIFUNC avopt65_data_TDP (ViSession instrumentHandle,
                                   ViReal64 *powerLeveldBm, ViInt16 *judgement);

ViStatus CVIFUNC avopt65_dataOnOffRatio (ViSession instrumentHandle,
                                          ViReal64 *ONLevel, ViReal64 *OFFLevel,
                                          ViReal64 *ONOFFRaito,
                                          ViInt16 *judgement);

ViStatus CVIFUNC avopt65_data_spuriTD (ViSession instrumentHandle,
                                       ViInt16 *tableSize,
                                       ViReal64 frequencyHz[], ViReal64 level[],
                                       ViInt16 judgement[]);

ViStatus CVIFUNC avopt65_data_FDP (ViSession instrumentHandle,
                                   ViReal64 *powerLeveldBm, ViInt16 *judgement);

ViStatus CVIFUNC avopt65_dataOBW (ViSession instrSession, ViPReal64 OBWHz,
                				  ViPReal64 lowFreqHz, ViPReal64 hiFreqHz,
                				  ViInt16 *judgement);

ViStatus CVIFUNC avopt65_data_duetoTRN (ViSession instrumentHandle,
                                        ViInt16 *numberofACP,
                                        ViReal64 ACPDatadBc[],
                                        ViInt16 judgement[]);

ViStatus CVIFUNC avopt65_data_duetoMOD (ViSession instrumentHandle,
                                        ViInt16 *numberofACP, ViReal64 ACPdBc[],
                                        ViInt16 ACPJudge[]);

ViStatus CVIFUNC avopt65_dataSpurMeas (ViSession instrumentHandle,
                                       ViInt16 *numberofPoint,
                                       ViReal64 frequencyHz[],
                                       ViReal64 leveldBm[], ViInt16 judge[]);

ViStatus CVIFUNC avopt65_data_spuriOB (ViSession instrumentHandle,
                                       ViInt16 *numberofPoint,
                                       ViReal64 frequencyHz[],
                                       ViReal64 leveldBm[], ViInt16 judgement[]);

ViStatus _VI_FUNC avopt65_dataCDPCoef (ViSession instrumentHandle,
                                       ViReal64 *timeAlignmentError,
                                       ViReal64 *carrierFrequencyErrorHz,
                                       ViReal64 *carrierFrequencyErrorPPM,
                                       ViReal64 *multipleRho, ViInt16 *PNOffset,
                                       ViReal64 *magnitudeError,
                                       ViReal64 *phaseError,
                                       ViReal64 *errorVectorMagnitude,
                                       ViReal64 *IQOriginOffset,
                                       ViReal64 *errorSignalPowerRatio,
                                       ViReal64 *txPower,
                                       ViReal64 *AVGPower_atChip,
                                       ViReal64 *peakCodeDomainError);

ViStatus _VI_FUNC avopt65_marker_dataCDPCoef (ViSession instrumentHandle,
                                              ViInt16 graphType,
                                              ViInt16 markerPosition,
                                              ViReal64 *markerData);

ViStatus _VI_FUNC avopt65_dataCDP (ViSession instrumentHandle,
                                   ViReal64 *estimatedRhoMultipleRho,
                                   ViReal64 *timeAlignmentError,
                                   ViReal64 *carrierFrequencyError,
                                   ViReal64 *magnitudeError,
                                   ViReal64 *phaseError,
                                   ViReal64 *errorVectorMagnitude,
                                   ViReal64 *IQOriginOffset, ViReal64 *PNDelay);

ViStatus _VI_FUNC avopt65_marker_dataCDP (ViSession instrumentHandle,
                                          ViInt16 viewChannel,
                                          ViReal64 *codePower, ViReal64 *rho,
                                          ViReal64 *powerdBm, ViReal64 *powerW,
                                          ViReal64 *EVM);

ViStatus _VI_FUNC avopt65_EVM_dataCDP (ViSession instrumentHandle,
                                       ViInt16 markerPosition, ViReal64 *EVM);

ViStatus _VI_FUNC avopt65_graphIQ_dataCDP (ViSession instrumentHandle,
                                           ViInt32 chipNumber,
                                           ViReal64 *IPhaseData,
                                           ViReal64 *QPhaseData);

ViStatus _VI_FUNC avopt65_graphError_dataCDP (ViSession instrumentHandle,
                                              ViInt16 markerPosition,
                                              ViInt16 *chipNumber,
                                              ViReal64 *errorData);

ViStatus CVIFUNC avopt65_dataTXPow (ViSession instrumentHandle,
                                     ViReal64 *powerLeveldBm,
                                     ViReal64 *powerLevelW,
                                     ViReal64 *peakFactordB);

ViStatus _VI_FUNC avopt65_dataCCDF (ViSession instrumentHandle,
                                    ViReal64 *peakFactor, ViReal64 *averagePower,
                                    ViReal64 *_10, ViReal64 *_1, ViReal64 *_01,
                                    ViReal64 *_001, ViReal64 *_0001,
                                    ViReal64 *_00001);

ViStatus CVIFUNC avopt65_close (ViSession instrSession);

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif

/*****************************************************************************/
/*=== END INCLUDE FILE ======================================================*/
/*****************************************************************************/

#endif
